@@ -48,31 +48,26 @@
  */
 #include <sys/types.h>
 
-/* A GNULIB replacement for this C99 header is supplied when it is missing.
+/* A GNULIB replacement for these C99 headers is supplied when any are missing.
  * See the comments in stdbool_.h for its limitations.
  */
 #include <stdbool.h>
+#include <stdint.h>
 
 /* Ditto for these POSIX.2 headers.  */
 #include <fnmatch.h>
 #include <getopt.h>	/* Has GNU extensions,  */
 
-/* We assume <sys/stat.h> because GNULIB does.  */
+/* We assume <sys/stat.h> & <unistd.h> because GNULIB does.  */
 #include <sys/stat.h>
+#include <unistd.h>
 
 #if !STDC_HEADERS && HAVE_MEMORY_H
 # include <memory.h>
 #endif /* !STDC_HEADERS && HAVE_MEMORY_H */
 #if HAVE_INTTYPES_H
 # include <inttypes.h>
-#else /* ! HAVE_INTTYPES_H */
-# if HAVE_STDINT_H
-#  include <stdint.h>
-# endif /* HAVE_STDINT_H */
 #endif /* HAVE_INTTYPES_H */
-#if HAVE_UNISTD_H
-# include <unistd.h>
-#endif /* HAVE_UNISTD_H */
 /* End the default set of autoconf includes */
 
 /* Assume these headers. */
@@ -188,15 +183,7 @@ int utime ();
 #else
 # define dirent direct
 # define NAMLEN(dirent) (dirent)->d_namlen
-# if HAVE_SYS_NDIR_H
-#  include <sys/ndir.h>
-# endif
-# if HAVE_SYS_DIR_H
-#  include <sys/dir.h>
-# endif
-# if HAVE_NDIR_H
-#  include <ndir.h>
-# endif
+# include <ndir.h>
 #endif
 
 /* Convert B 512-byte blocks to kilobytes if K is nonzero,
@@ -286,8 +273,13 @@ extern unsigned char WNT_filename_classes[];
    * WOE32 needs its own class since \ and C:\ style absolute paths also need
    * to be accounted for.
    */
+#  if defined(USE_VMS_FILENAMES)
+#   define FOLD_FN_CHAR(c) (VMS_filename_classes[(unsigned char) (c)])
+extern unsigned char VMS_filename_classes[];
+#  else
 #   define FOLD_FN_CHAR(c) (OSX_filename_classes[(unsigned char) (c)])
 extern unsigned char OSX_filename_classes[];
+#  endif
 # endif /* __CYGWIN32__ || WOE32 */
 
 /* The following need to be declared for all case insensitive filesystems.
