@@ -1,6 +1,11 @@
 /*
- * Copyright (c) 1992, Brian Berliner and Jeff Polk
- * Copyright (c) 1989-1992, Brian Berliner
+ * Copyright (C) 1986-2006 The Free Software Foundation, Inc.
+ *
+ * Portions Copyright (C) 1998-2005 Derek Price, Ximbiot <http://ximbiot.com>,
+ *                                  and others.
+ *
+ * Portions Copyright (c) 1992, Brian Berliner and Jeff Polk
+ * Portions Copyright (c) 1989-1992, Brian Berliner
  * 
  * You may distribute under the terms of the GNU General Public License as
  * specified in the README file that comes with the CVS source distribution.
@@ -9,11 +14,20 @@
  * 
  */
 
-#include "cvs.h"
-#ifdef CVS_ADMIN_GROUP
-#include <grp.h>
+#ifdef HAVE_CONFIG_H
+# include <config.h>
 #endif
 
+/* CVS headers.  */
+#include "ignore.h"
+#include "recurse.h"
+#include "repos.h"
+#include "wrapper.h"
+
+#include "cvs.h"
+
+
+
 static Dtype admin_dirproc (void *callerdat, const char *dir,
                             const char *repos, const char *update_dir,
                             List *entries);
@@ -60,6 +74,8 @@ static const char *const admin_usage[] =
     "\t-u[rev]    Unlock the revision (latest revision on branch,\n",
     "\t           latest revision on trunk if omitted).\n",
     "\t-U         Unset strict locking.\n",
+    "\t--execute    Turn on execute bits on repository file.\n",
+    "\t--no-execute Turn off execute bits on repository file.\n",
     "(Specify the --help global option for a list of other help options)\n",
     NULL
 };
@@ -96,6 +112,8 @@ struct admin_data
     /* Interactive (-I).  Problematic with client/server.  */
     int interactive;
 
+    enum {AVOID = 0, NOEXECUTE, EXECUTE} execute;
+
     /* This is the cheesy part.  It is a vector with the options which
        we don't deal with above (e.g. "-afoo" "-abar,baz").  In the future
        this presumably will be replaced by other variables which break
@@ -104,6 +122,10 @@ struct admin_data
     int ac;
     char **av;
     int av_alloc;
+
+    /* This contains a printable version of the command line used
+       for logging. */
+    char *cmdline;
 };
 
 /* Add an argument.  OPT is the option letter, e.g. 'a'.  ARG is the
@@ -113,24 +135,17 @@ struct admin_data
 static void
 arg_add (struct admin_data *dat, int opt, char *arg)
 {
-    char *newelt = xmalloc ((arg == NULL ? 0 : strlen (arg)) + 3);
-    strcpy (newelt, "-");
-    newelt[1] = opt;
-    if (arg == NULL)
-	newelt[2] = '\0';
-    else
-	strcpy (newelt + 2, arg);
+    char *newelt = Xasprintf ("-%c%s", opt, arg ? arg : "");
 
     if (dat->av_alloc == 0)
     {
 	dat->av_alloc = 1;
-	dat->av = (char **) xmalloc (dat->av_alloc * sizeof (*dat->av));
+	dat->av = xnmalloc (dat->av_alloc, sizeof (*dat->av));
     }
     else if (dat->ac >= dat->av_alloc)
     {
 	dat->av_alloc *= 2;
-	dat->av = (char **) xrealloc (dat->av,
-				      dat->av_alloc * sizeof (*dat->av));
+	dat->av = xnrealloc (dat->av, dat->av_alloc, sizeof (*dat->av));
     }
     dat->av[dat->ac++] = newelt;
 }
@@ -153,6 +168,11 @@ postadmin_proc (const char *repository, const char *filter, void *closure)
      * %p = shortrepos
      * %r = repository
      */
+    /*
+     * Cast any NULL arguments as appropriate pointers as this is an
+     * stdarg function and we need to be certain the caller gets what
+     * is expected.
+     */
     cmdline = format_cmdline (
 #ifdef SUPPORT_OLD_INFO_FMT_STRINGS
 	                      false, srepos,
@@ -164,8 +184,7 @@ postadmin_proc (const char *repository, const char *filter, void *closure)
 #endif /* SERVER_SUPPORT */
 	                      "p", "s", srepos,
 	                      "r", "s", current_parsed_root->directory,
-	                      (char *)NULL
-	                     );
+	                      (char *) NULL);
 
     if (!cmdline || !strlen (cmdline))
     {
@@ -204,14 +223,192 @@ admin_filesdoneproc (void *callerdat, int err, const char *repository,
 
 
 
+static const char short_options[] =
+    "+ib::c:a:A:e::l::u::LUn:N:m:o:s:t::IqxV:k:";
+
+enum {OPT_NONE = 0, OPT_EXECUTE, OPT_NOEXECUTE} opt_values;
+static struct option long_options[] =
+{
+    {"execute", 0, NULL, OPT_EXECUTE},
+    {"no-execute", 0, NULL, OPT_NOEXECUTE},
+    {0, 0, NULL, OPT_NONE}
+};
+
+
+
+/* Accept a `;' delimited string and break it into tokens.  Allocate a
+ * return string.  Copy the first token into the return string
+ * checking to be sure that each character is a valid option character
+ * of the short_options string. For remaining tokens, convert to the
+ * long option VAL (from the global LONG_OPTIONS above) and append
+ * that char to the return value.  When long option tokens are
+ * unrecognized, a warning is printed and they are ignored.
+ *
+ * i.e., S will be of the format `[SHORTOPTIONS][;LONGOPTION]...'.  It is
+ * perfectly acceptable for SHORTOPTIONS to resolve to the empty string, 
+ * an empty LONGOPTION will also be ignored.
+ */
+char *
+make_UserAdminOptions (const char *infopath, unsigned int ln, const char *s)
+{
+    const char *cur_opt, *next_opt;
+    size_t len;
+    char *ns;
+
+    assert (s);
+
+    cur_opt = s;
+
+    next_opt = strchr (cur_opt, ';');
+    if (next_opt)
+	len = next_opt - cur_opt;
+    else
+	len = strlen (cur_opt);
+
+    ns = xmalloc (len + 1);
+    *ns = '\0';
+    if (len > 0)
+    {
+	const char *p;
+	size_t nspos = 0;
+	/* validate short options */
+	for (p = cur_opt; p < (cur_opt + len); p++)
+	{
+	    if (*p == '+' || *p == ':' || strchr (short_options, *p) == NULL)
+		error (0, 0,
+		       "%s [%u]: Unrecognized short admin option `%c'.",
+		       infopath, ln, *p);
+	    else
+		ns[nspos++] = *p;
+	}
+	ns[nspos] = '\0';
+	if (nspos > 0)
+	    TRACE (TRACE_FUNCTION, "Setting short UserAdminOptions `%s'", ns);
+    }
+
+    /* process long options (if any) */
+    while ((cur_opt = next_opt))
+    {
+	next_opt = strchr (++cur_opt, ';');
+
+	if (next_opt)
+	    len = next_opt - cur_opt;
+	else
+	    len = strlen (cur_opt);
+
+	/* ignore empty long options (ie, ';;') */
+	if (len > 0)
+	{
+	    struct option *found;
+
+	    for (found = long_options; found->name; found++)
+		if (len == strlen (found->name)
+		    && !strncmp (cur_opt, found->name, len))
+		    break;
+
+	    if (found->name)
+	    {
+		size_t nslen = strlen (ns);
+
+		assert (found->val);
+
+		ns = xrealloc (ns, nslen + 2);
+		ns[nslen++] = found->val;
+		ns[nslen] = '\0';
+		TRACE (TRACE_FUNCTION, "Adding long UserAdminOptions `%s'",
+		       found->name);
+	    }
+	    else
+	    {
+		char *tmp = xmalloc (len + 1);
+		strncpy (tmp, cur_opt, len);
+		tmp[len] = '\0';
+		error (0, 0,
+		       "%s [%u]: Unrecognized long admin option `%s'.",
+		       infopath, ln, tmp);
+		free (tmp);
+	    }
+	}
+    }
+
+    return ns;
+}
+
+
+
+static size_t
+wescape(char *dst, const char *src)
+{
+    const unsigned char *s = (const unsigned char *)src;
+    char *d = dst;
+    for (; *s; s++) {
+	if (!isprint (*s) || isspace (*s) || *s == '|') {
+	    /* These next four statements are the same as:
+	     *   d += sprintf(d, "\\%03o", *s);
+	     * but do not make assumptions that sprintf does the right
+	     * thing with regard to the integer it returns.
+	     */
+	    /* Convert special characters to octal */
+	    *d++ = '\\';
+	    *d++ = ((*s >> 6) & 3) + '0';
+	    *d++ = ((*s >> 3) & 7) + '0';
+	    *d++ = ((*s >> 0) & 7) + '0';
+	} else  {
+	    *d++ = *s;
+	}
+    }
+    *d = '\0';
+    return d - dst;
+}
+
+
+
+static char *
+makecmdline (int argc, char **argv)
+{
+    size_t clen = 1024, wlen = 1024, len, cpos = 0, i;
+    char *cmd = xmalloc (clen);
+    char *word = xmalloc (wlen);
+
+    for (i = 0; i < argc; i++) {
+	char *arg = argv[i];
+	if (i == 0)
+	{
+	    if (strncmp (argv[i], "cvs server", 10) == 0)
+		arg = "admin";
+	    else if (strncmp (argv[i], "cvs ", 4) == 0)
+		arg = argv[i] + 4;
+	}
+
+	len = strlen (arg);
+	if (len * 4 < wlen) {
+	    wlen += len * 4;
+	    word = xrealloc (word, wlen);
+	}
+	len = wescape (word, arg);
+	if (clen - cpos < len + 2) {
+	    clen += len + 2;
+	    cmd = xrealloc (cmd, clen);
+	}
+	memcpy (&cmd[cpos], word, len);
+	cpos += len;
+	cmd[cpos++] = ' ';
+    }
+    if (cpos != 0)
+	cmd[cpos - 1] = '\0';
+    else
+	cmd[cpos] = '\0';
+    free (word);
+
+    return cmd;
+}
+
+
+
 int
 admin (int argc, char **argv)
 {
     int err;
-#ifdef CVS_ADMIN_GROUP
-    struct group *grp;
-    struct group *getgrnam(const char *);
-#endif
     struct admin_data admin_data;
     int c;
     int i;
@@ -223,14 +420,16 @@ admin (int argc, char **argv)
     wrap_setup ();
 
     memset (&admin_data, 0, sizeof admin_data);
+    admin_data.cmdline = makecmdline (argc, argv);
 
     /* TODO: get rid of `-' switch notation in admin_data.  For
        example, admin_data->branch should be not `-bfoo' but simply `foo'. */
 
     optind = 0;
     only_allowed_options = true;
-    while ((c = getopt (argc, argv,
-			"+ib::c:a:A:e::l::u::LUn:N:m:o:s:t::IqxV:k:")) != -1)
+    while ((c = getopt_long
+	    (argc, argv, short_options, long_options, NULL))
+	   != EOF)
     {
 	if (
 # ifdef CLIENT_SUPPORT
@@ -242,6 +441,14 @@ admin (int argc, char **argv)
 
 	switch (c)
 	{
+	    case OPT_EXECUTE:	/* --execute */
+		admin_data.execute = EXECUTE;
+		break;
+
+	    case OPT_NOEXECUTE:	/* --no-execute */
+		admin_data.execute = NOEXECUTE;
+		break;
+
 	    case 'i':
 		/* This has always been documented as useless in cvs.texinfo
 		   and it really is--admin_fileproc silently does nothing
@@ -259,11 +466,7 @@ admin (int argc, char **argv)
 		if (optarg == NULL)
 		    admin_data.branch = xstrdup ("-b");
 		else
-		{
-		    admin_data.branch = xmalloc (strlen (optarg) + 5);
-		    strcpy (admin_data.branch, "-b");
-		    strcat (admin_data.branch, optarg);
-		}
+		    admin_data.branch = Xasprintf ("-b%s", optarg);
 		break;
 
 	    case 'c':
@@ -272,9 +475,7 @@ admin (int argc, char **argv)
 		    error (0, 0, "duplicate 'c' option");
 		    goto usage_error;
 		}
-		admin_data.comment = xmalloc (strlen (optarg) + 5);
-		strcpy (admin_data.comment, "-c");
-		strcat (admin_data.comment, optarg);
+		admin_data.comment = Xasprintf ("-c%s", optarg);
 		break;
 
 	    case 'a':
@@ -366,9 +567,7 @@ admin (int argc, char **argv)
 		    error (0, 0, "duplicate '-o' option");
 		    goto usage_error;
 		}
-		admin_data.delete_revs = xmalloc (strlen (optarg) + 5);
-		strcpy (admin_data.delete_revs, "-o");
-		strcat (admin_data.delete_revs, optarg);
+		admin_data.delete_revs = Xasprintf ("-o%s", optarg);
 		break;
 
 	    case 's':
@@ -443,53 +642,18 @@ admin (int argc, char **argv)
     argc -= optind;
     argv += optind;
 
-#ifdef CVS_ADMIN_GROUP
     /* The use of `cvs admin -k' is unrestricted.  However, any other
        option is restricted if the group CVS_ADMIN_GROUP exists on the
        server.  */
-    if (
-# ifdef CLIENT_SUPPORT
-        /* This is only "secure" on the server, since the user could edit the
-	 * RCS file on a local host, but some people like this kind of
-	 * check anyhow.  The alternative would be to check only when
-	 * (server_active) rather than when not on the client.
-	 */
-        !current_parsed_root->isremote &&
-# endif	/* CLIENT_SUPPORT */
-        !only_allowed_options
-	&& (grp = getgrnam (CVS_ADMIN_GROUP)) != NULL)
-    {
-#ifdef HAVE_GETGROUPS
-	gid_t *grps;
-	int n;
-
-	/* get number of auxiliary groups */
-	n = getgroups (0, NULL);
-	if (n < 0)
-	    error (1, errno, "unable to get number of auxiliary groups");
-	grps = xmalloc ((n + 1) * sizeof *grps);
-	n = getgroups (n, grps);
-	if (n < 0)
-	    error (1, errno, "unable to get list of auxiliary groups");
-	grps[n] = getgid();
-	for (i = 0; i <= n; i++)
-	    if (grps[i] == grp->gr_gid) break;
-	free (grps);
-	if (i > n)
-	    error (1, 0, "usage is restricted to members of the group %s",
-		   CVS_ADMIN_GROUP);
-#else
-	char *me = getcaller();
-	char **grnam;
-	
-	for (grnam = grp->gr_mem; *grnam; grnam++)
-	    if (strcmp (*grnam, me) == 0) break;
-	if (!*grnam && getgid() != grp->gr_gid)
-	    error (1, 0, "usage is restricted to members of the group %s",
-		   CVS_ADMIN_GROUP);
-#endif
-    }
-#endif /* defined CVS_ADMIN_GROUP */
+    /* This is only "secure" on the server, since the user could edit the
+     * RCS file on a local host, but some people like this kind of
+     * check anyhow.  The alternative would be to check only when
+     * (server_active) rather than when not on the client.
+     */
+    if (!current_parsed_root->isremote && !only_allowed_options &&
+ 	!is_admin ())
+ 	error (1, 0, "usage is restricted to members of the group %s",
+ 	       CVS_ADMIN_GROUP);
 
     for (i = 0; i < admin_data.ac; ++i)
     {
@@ -542,6 +706,10 @@ admin (int argc, char **argv)
 	    send_arg ("-U");
 	if (admin_data.delete_revs != NULL)
 	    send_arg (admin_data.delete_revs);
+	if (admin_data.execute == EXECUTE)
+	    send_arg ("--execute");
+	else if (admin_data.execute == NOEXECUTE)
+	    send_arg ("--no-execute");
 	if (admin_data.desc != NULL)
 	{
 	    char *p = admin_data.desc;
@@ -593,7 +761,10 @@ admin (int argc, char **argv)
 
     Lock_Cleanup ();
 
+/* This just suppresses a warning from -Wall.  */
+#ifdef CLIENT_SUPPORT
  return_it:
+#endif /* CLIENT_SUPPORT */
     if (admin_data.branch != NULL)
 	free (admin_data.branch);
     if (admin_data.comment != NULL)
@@ -638,6 +809,9 @@ admin_fileproc (void *callerdat, struct file_info *finfo)
 	goto exitfunc;
     }
 
+    history_write ('X', finfo->update_dir, admin_data->cmdline, finfo->file,
+		   finfo->repository);
+
     rcs = vers->srcfile;
     if (rcs == NULL)
     {
@@ -648,7 +822,7 @@ admin_fileproc (void *callerdat, struct file_info *finfo)
     }
 
     if (rcs->flags & PARTIAL)
-	RCS_reparsercsfile (rcs, (FILE **) NULL, (struct rcsbuffer *) NULL);
+	RCS_reparsercsfile (rcs, NULL, NULL);
 
     if (!really_quiet)
     {
@@ -885,6 +1059,13 @@ admin_fileproc (void *callerdat, struct file_info *finfo)
 		{
 		    tag = xstrdup (arg + 2);
 		    rev = RCS_head (rcs);
+		    if (!rev)
+		    {
+			error (0, 0, "No head revision in archive file `%s'.",
+			       rcs->path);
+			status = 1;
+			continue;
+		    }
 		}
 		else
 		{
@@ -954,8 +1135,8 @@ admin_fileproc (void *callerdat, struct file_info *finfo)
 		delta = n->data;
 		if (delta->text == NULL)
 		{
-		    delta->text = (Deltatext *) xmalloc (sizeof (Deltatext));
-		    memset ((void *) delta->text, 0, sizeof (Deltatext));
+		    delta->text = xmalloc (sizeof (Deltatext));
+		    memset (delta->text, 0, sizeof (Deltatext));
 		}
 		delta->text->version = xstrdup (delta->version);
 		delta->text->log = make_message_rcsvalid (msg);
@@ -974,6 +1155,48 @@ admin_fileproc (void *callerdat, struct file_info *finfo)
     if (status == 0)
     {
 	RCS_rewrite (rcs, NULL, NULL);
+
+        /*
+         * Update the execute bit for the file if requested.
+         */
+        if (admin_data->execute != AVOID)
+        {
+            struct stat sb;
+ 
+            if (stat (rcs->path, &sb) < 0)
+                error (0, errno, "cannot stat `%s'", rcs->path);
+            else
+            {
+                mode_t mode;
+ 
+                if (admin_data->execute == EXECUTE)
+                    mode = (sb.st_mode
+                            | (((sb.st_mode & S_IRUSR) ? S_IXUSR : 0)
+                                  | ((sb.st_mode & S_IRGRP) ? S_IXGRP : 0)
+                                  | ((sb.st_mode & S_IROTH) ? S_IXOTH : 0)));
+                else /* admin_data->execute == NOEXECUTE */
+                    mode = (sb.st_mode
+                            & ~(S_IEXEC | S_IXGRP | S_IXOTH));
+ 
+                if (mode == sb.st_mode)
+                    error (0, 0, "%s: already has mode=0%o",
+			   rcs->path, (unsigned int) mode);
+                else
+                {
+                    TRACE (TRACE_FLOW, "chmod(%s,0%o)", rcs->path,
+                           (unsigned int) mode);
+                    
+		    if (!noexec)
+		    {
+			if (chmod (rcs->path, mode) < 0)
+			    error (0, errno,
+				   "cannot change mode of file `%s'",
+				   rcs->path);
+		    }
+                }
+            }
+        }
+
 	if (!really_quiet)
 	    cvs_output ("done\n", 5);
     }
