@@ -1,22 +1,30 @@
+/*
+ * Copyright (C) 1996-2006 The Free Software Foundation, Inc.
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2, or (at your option)
+ * any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ */
+
 /* Code for the buffer data structure.  */
 
-#include "cvs.h"
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
+
+/* Verify interface.  */
 #include "buffer.h"
-#include "pagealign_alloc.h"
 
-#if defined (SERVER_SUPPORT) || defined (CLIENT_SUPPORT)
+/* CVS headers.  */
+#include "cvs.h"
 
-# ifdef HAVE_WINSOCK_H
-#   include <winsock.h>
-# else
-#  include <sys/socket.h>
-# endif
 
-/* OS/2 doesn't have EIO.  FIXME: this whole notion of turning
-   a different error into EIO strikes me as pretty dubious.  */
-# if !defined( EIO )
-#   define EIO EBADPOS
-# endif
 
 /* Local functions.  */
 static void buf_default_memory_error (struct buffer *);
@@ -26,13 +34,13 @@ static struct buffer_data *get_buffer_data (void);
 
 /* Initialize a buffer structure.  */
 struct buffer *
-buf_initialize (int (*input) (void *, char *, size_t, size_t, size_t *),
-	        int (*output) (void *, const char *, size_t, size_t *),
-                int (*flush) (void *),
-                int (*block) (void *, bool),
-                int (*get_fd) (void *),
-                int (*shutdown) (struct buffer *),
-                void (*memory) (struct buffer *),
+buf_initialize (type_buf_input input,
+                type_buf_output output,
+                type_buf_flush flush,
+                type_buf_block block,
+                type_buf_get_fd get_fd,
+                type_buf_shutdown shutdown,
+                type_buf_memory_error memory_error,
                 void *closure)
 {
     struct buffer *buf;
@@ -47,7 +55,7 @@ buf_initialize (int (*input) (void *, char *, size_t, size_t, size_t *),
     buf->block = block;
     buf->get_fd = get_fd;
     buf->shutdown = shutdown;
-    buf->memory_error = memory ? memory : buf_default_memory_error;
+    buf->memory_error = memory_error ? memory_error : buf_default_memory_error;
     buf->closure = closure;
     return buf;
 }
@@ -95,7 +103,7 @@ get_buffer_data (void)
     struct buffer_data *ret;
 
     ret = xmalloc (sizeof (struct buffer_data));
-    ret->text = pagealign_xalloc (BUFFER_DATA_SIZE);
+    ret->text = xmalloc (BUFFER_DATA_SIZE);
 
     return ret;
 }
@@ -130,7 +138,7 @@ buf_empty_p (struct buffer *buf)
 
 
 
-# if defined (SERVER_FLOWCONTROL) || defined (PROXY_SUPPORT)
+#if defined (SERVER_FLOWCONTROL) || defined (PROXY_SUPPORT)
 /*
  * Count how much data is stored in the buffer..
  * Note that each buffer is a xmalloc'ed chunk BUFFER_DATA_SIZE.
@@ -146,7 +154,7 @@ buf_count_mem (struct buffer *buf)
 
     return mem;
 }
-# endif /* SERVER_FLOWCONTROL || PROXY_SUPPORT */
+#endif /* SERVER_FLOWCONTROL || PROXY_SUPPORT */
 
 
 
@@ -246,7 +254,7 @@ buf_free_datas (struct buffer_data *first, struct buffer_data *last)
     {
 	p = b;
 	n = b->next;
-	pagealign_free (b->text);
+	free (b->text);
 	free (b);
 	b = n;
     } while (p != last);
@@ -483,7 +491,7 @@ buf_append_data (struct buffer *buf, struct buffer_data *data,
 
 
 
-# ifdef PROXY_SUPPORT
+#ifdef PROXY_SUPPORT
 /* Copy data structures and append them to a buffer.
  *
  * ERRORS
@@ -518,7 +526,7 @@ buf_copy_data (struct buffer *buf, struct buffer_data *data,
 
     buf_append_data (buf, first, new);
 }
-# endif /* PROXY_SUPPORT */
+#endif /* PROXY_SUPPORT */
 
 
 
@@ -941,8 +949,6 @@ buf_read_short_line (struct buffer *buf, char **line, size_t *lenp,
 int
 buf_read_data (struct buffer *buf, size_t want, char **retdata, size_t *got)
 {
-    assert (buf->input != NULL);
-
     while (buf->data != NULL && buf->data->size == 0)
     {
 	struct buffer_data *next;
@@ -960,6 +966,12 @@ buf_read_data (struct buffer *buf, size_t want, char **retdata, size_t *got)
 	int status;
 	size_t get, nbytes;
 
+	if (!buf->input)
+	    /* nonio (memory) buffers have no underlying input methods.  If
+	     * there are no buffer datas, just return EOF.
+	     */
+	    return -1;
+
 	data = get_buffer_data ();
 	if (data == NULL)
 	{
@@ -1252,6 +1264,18 @@ buf_shutdown (struct buffer *buf)
 
 
 
+#if defined (SERVER_SUPPORT) || defined (CLIENT_SUPPORT)
+
+# include <sys/socket.h>
+
+/* OS/2 doesn't have EIO.  FIXME: this whole notion of turning
+   a different error into EIO strikes me as pretty dubious.  */
+# if !defined (EIO)
+#   define EIO EBADPOS
+# endif
+
+
+
 /* Certain types of communication input and output data in packets,
    where each packet is translated in some fashion.  The packetizing
    buffer type supports that, given a buffer which handles lower level
@@ -1645,6 +1669,11 @@ packetizing_buffer_output (void *closure, const char *data, size_t have,
 	buf_output (pb->buf, outbuf, translated + 2);
     else
     {
+	/* if ((have + PACKET_SLOP + 4) > BUFFER_DATA_SIZE), then
+	   outdata may be NULL. */
+	if (outdata == NULL)
+	    abort ();
+
 	outdata->size = translated + 2;
 	buf_append_data (pb->buf, outdata, outdata);
     }
@@ -1719,6 +1748,8 @@ struct fd_buffer
     int fd;
     /* Nonzero if the file descriptor is in blocking mode.  */
     int blocking;
+    /* Time to wait for reads and writes to succeed.  */
+    long timeout;
     /* The child process id when fd is a pipe.  */
     pid_t child_pid;
     /* The connection info, when fd is a pipe to a server.  */
@@ -1733,18 +1764,19 @@ static int fd_buffer_get_fd (void *);
 static int fd_buffer_shutdown (struct buffer *);
 
 /* Initialize a buffer built on a file descriptor.  FD is the file
-   descriptor.  INPUT is nonzero if this is for input, zero if this is
+   descriptor.  INPUT is true if this is for input, false if this is
    for output.  MEMORY is the function to call when a memory error
    occurs.  */
 
 struct buffer *
 fd_buffer_initialize (int fd, pid_t child_pid, cvsroot_t *root, bool input,
-                      void (*memory) (struct buffer *))
+                      long timeout, void (*memory) (struct buffer *))
 {
     struct fd_buffer *n;
 
     n = xmalloc (sizeof *n);
     n->fd = fd;
+    n->timeout = timeout;
     n->child_pid = child_pid;
     n->root = root;
     fd_buffer_block (n, true);
@@ -1761,7 +1793,7 @@ fd_buffer_initialize (int fd, pid_t child_pid, cvsroot_t *root, bool input,
 
 /* The buffer input function for a buffer built on a file descriptor.
  *
- * In non-blocing mode, this function will read as many bytes as it can in a
+ * In non-blocking mode, this function will read as many bytes as it can in a
  * single try, up to SIZE bytes, and return.
  *
  * In blocking mode with NEED > 0, this function will read as many bytes as it
@@ -1788,6 +1820,7 @@ fd_buffer_initialize (int fd, pid_t child_pid, cvsroot_t *root, bool input,
  *
  * RETURNS
  *   errno	On error.
+ *   -3		On timeout.
  *   -1		On EOF.
  *   0		Otherwise.
  *
@@ -1800,7 +1833,7 @@ fd_buffer_input (void *closure, char *data, size_t need, size_t size,
 		 size_t *got)
 {
     struct fd_buffer *fb = closure;
-    int nbytes;
+    ssize_t nbytes;
 
     assert (need <= size);
 
@@ -1810,10 +1843,14 @@ fd_buffer_input (void *closure, char *data, size_t need, size_t size,
     {
 	int status;
 	fd_set readfds;
+	struct timeval timeout;
+	struct timeval *timeout_p;
 
 	/* Set non-block.  */
         status = fd_buffer_block (fb, false);
 	if (status != 0) return status;
+	if (fb->timeout) timeout_p = &timeout;
+	else timeout_p = NULL;
 
 	FD_ZERO (&readfds);
 	FD_SET (fb->fd, &readfds);
@@ -1821,14 +1858,28 @@ fd_buffer_input (void *closure, char *data, size_t need, size_t size,
 	{
 	    int numfds;
 
-	    do {
+	    do
+	    {
+		if (timeout_p)
+		{
+		    timeout.tv_sec = fb->timeout;
+		    timeout.tv_usec = 0;
+		}
+
 		/* This used to select on exceptions too, but as far
 		   as I know there was never any reason to do that and
 		   SCO doesn't let you select on exceptions on pipes.  */
-		numfds = select (fb->fd + 1, &readfds, NULL, NULL, NULL);
+		numfds = fd_select (fb->fd + 1, &readfds, NULL, NULL,
+				    timeout_p);
 		if (numfds < 0 && errno != EINTR)
 		{
 		    status = errno;
+		    assert (status);
+		    goto block_done;
+		}
+		else if (numfds == 0)
+		{
+		    status = -3;
 		    goto block_done;
 		}
 	    } while (numfds < 0);
@@ -1846,13 +1897,13 @@ fd_buffer_input (void *closure, char *data, size_t need, size_t size,
 		     * on the fact that we will read EOF again next time.
 		     */
 		    status = 0;
-		    break;
+		    goto block_done;
 		}
 		else
 		{
 		    /* Return EOF.  */
 		    status = -1;
-		    break;
+		    goto block_done;
 		}
 	    }
 
@@ -1862,12 +1913,13 @@ fd_buffer_input (void *closure, char *data, size_t need, size_t size,
 		if (!blocking_error (errno))
 		{
 		    status = errno;
-		    break;
+		    assert (status);
+		    goto block_done;
 		}
 		/* else Everything's fine, we just didn't get any data.  */
 	    }
-
-	    *got += nbytes;
+	    else
+		*got += nbytes;
 	} while (*got < need);
 
 block_done:
@@ -1908,44 +1960,103 @@ block_done:
 
 
 /* The buffer output function for a buffer built on a file descriptor.  */
-
 static int
 fd_buffer_output (void *closure, const char *data, size_t have, size_t *wrote)
 {
-    struct fd_buffer *fd = closure;
+    struct fd_buffer *fb = closure;
+    ssize_t nbytes;
 
     *wrote = 0;
 
-    while (have > 0)
+    if (fb->blocking)
     {
-	int nbytes;
+	int status;
+	fd_set writefds;
+	struct timeval timeout;
+	struct timeval *timeout_p;
 
-	nbytes = write (fd->fd, data, have);
+	/* Set non-block.  */
+        status = fd_buffer_block (fb, false);
+	if (status != 0) return status;
+	if (fb->timeout) timeout_p = &timeout;
+	else timeout_p = NULL;
 
-	if (nbytes <= 0)
+	FD_ZERO (&writefds);
+	FD_SET (fb->fd, &writefds);
+	do
 	{
-	    if (! fd->blocking
-		&& (nbytes == 0 || blocking_error (errno)))
+	    int numfds;
+
+	    do
 	    {
-		/* A nonblocking write failed to write any data.  Just
-		   return.  */
-		return 0;
+		if (timeout_p)
+		{
+		    timeout.tv_sec = fb->timeout;
+		    timeout.tv_usec = 0;
+		}
+
+		/* This used to select on exceptions too, but as far
+		   as I know there was never any reason to do that and
+		   SCO doesn't let you select on exceptions on pipes.  */
+		numfds = fd_select (fb->fd + 1, NULL, &writefds, NULL,
+				    timeout_p);
+		if (numfds < 0 && errno != EINTR)
+		{
+		    status = errno;
+		    assert (status);
+		    goto block_done;
+		}
+		else if (numfds == 0)
+		{
+		    status = -3;
+		    goto block_done;
+		}
+	    } while (numfds < 0);
+
+	    nbytes = write (fb->fd, data + *wrote, have - *wrote);
+
+	    if (nbytes < 0)
+	    {
+		/* Some error occurred.  */
+		if (!blocking_error (errno))
+		{
+		    status = errno;
+		    assert (status);
+		    goto block_done;
+		}
+		/* else Everything's fine, we just didn't get any data.  */
 	    }
+	    else
+		*wrote += nbytes;
+	} while (*wrote < have);
+	assert (*wrote == have);
 
-	    /* Some sort of error occurred.  */
+block_done:
+	if (status == 0)
+	{
+	    int newstatus;
 
-	    if (nbytes == 0)
-		return EIO;
-
-	    return errno;
+	    /* OK - Reset block.  */
+	    newstatus = fd_buffer_block (fb, true);
+	    if (newstatus) status = newstatus;
 	}
-
-	*wrote += nbytes;
-	data += nbytes;
-	have -= nbytes;
+	return status;
     }
 
-    return 0;
+    /* The above will always return.  Handle non-blocking read.  */
+    nbytes = write (fb->fd, data, have);
+
+    if (nbytes >= 0)
+    {
+	*wrote = nbytes;
+	return 0;
+    }
+
+    if (blocking_error (errno))
+	/* Everything's fine, we just didn't get any data.  */
+	return 0;
+
+    return errno;
 }
 
 
@@ -1998,7 +2109,7 @@ fd_buffer_block (void *closure, bool block)
 	bool isdevnull = false;
 
 	if (devnull_set == -1)
-	    devnull_set = stat ("/dev/null", &devnull);
+	    devnull_set = stat (DEVNULL, &devnull);
 
 	if (devnull_set >= 0)
 	    /* Equivalent to /dev/null ? */
@@ -2108,7 +2219,7 @@ fd_buffer_shutdown (struct buffer *buf)
 	 */
 	else if (fb->root && pclose (fb->fd) == EOF)
 	{
-	    error (1, errno, "closing connection to %s",
+	    error (0, errno, "closing connection to %s",
 		   fb->root->hostname);
 	    closefd = false;
 	}
@@ -2143,22 +2254,21 @@ fd_buffer_shutdown (struct buffer *buf)
 
     if (statted && closefd && close (fb->fd) == -1)
     {
-	if (0
-# ifdef SERVER_SUPPORT
-	    || server_active
-# endif /* SERVER_SUPPORT */
-           )
+	if (server_active)
 	{
             /* Syslog this? */
 	}
 # ifdef CLIENT_SUPPORT
+	/* We are already closing the connection.
+	 * On error, print a warning and try to
+	 * continue to avoid infinte loops.
+	 */
 	else if (fb->root)
-            error (1, errno, "closing down connection to %s",
+            error (0, errno, "closing down connection to %s",
                    fb->root->hostname);
-	    /* EXITS */
 # endif /* CLIENT_SUPPORT */
-
-	error (0, errno, "closing down buffer");
+	else
+	    error (0, errno, "closing down buffer");
     }
 
     /* If we were talking to a process, make sure it exited */
@@ -2169,8 +2279,13 @@ fd_buffer_shutdown (struct buffer *buf)
 	do
 	    w = waitpid (fb->child_pid, NULL, 0);
 	while (w == -1 && errno == EINTR);
+
+	/* We are already closing the connection.
+	 * On error, print a warning and try to
+	 * continue to avoid infinte loops.
+	 */
 	if (w == -1)
-	    error (1, errno, "waiting for process %d", fb->child_pid);
+	    error (0, errno, "waiting for process %d", fb->child_pid);
     }
 
     free (buf->closure);
