@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1986-2005 The Free Software Foundation, Inc.
+ * Copyright (C) 1986-2006 The Free Software Foundation, Inc.
  *
  * Portions Copyright (C) 1998-2005 Derek Price, Ximbiot <http://ximbiot.com>,
  *                                  and others.
@@ -12,6 +12,16 @@
  * 
  */
 
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
+
+/* Verify interface.  */
+#include "classify.h"
+
+/* CVS Headers.  */
+#include "no_diff.h"
+
 #include "cvs.h"
 
 static void sticky_ck (struct file_info *finfo, int aflag,
@@ -43,6 +53,10 @@ Classify_File (struct file_info *finfo, char *tag, char *date, char *options,
     Vers_TS *vers;
     Ctype ret;
 
+    TRACE (TRACE_FUNCTION, "classify_file (%s, %s, %s, %s)",
+	   finfo->fullname, tag ? tag : "(null)", date ? date : "(null)",
+	   options ? options : "(null)");
+
     /* get all kinds of good data about the file */
     vers = Version_TS (finfo, options, tag, date,
 		       force_tag_match, 0);
@@ -236,19 +250,8 @@ Classify_File (struct file_info *finfo, char *tag, char *date, char *options,
 			   finfo->fullname);
 		ret = T_REMOVE_ENTRY;
 	    }
-	    else if (strcmp (vers->ts_user, vers->ts_rcs) == 0)
-	    {
-
-		/*
-		 * The user file is still unmodified, so just remove it from
-		 * the entry list
-		 */
-		if (!really_quiet)
-		    error (0, 0, "`%s' is no longer in the repository",
-			   finfo->fullname);
-		ret = T_REMOVE_ENTRY;
-	    }
-	    else if (No_Difference (finfo, vers))
+	    else if (strcmp (vers->ts_user, vers->ts_rcs)
+		     && No_Difference (finfo, vers))
 	    {
 		/* they are different -> conflict */
 		if (!really_quiet)
@@ -260,10 +263,13 @@ Classify_File (struct file_info *finfo, char *tag, char *date, char *options,
 	    }
 	    else
 	    {
-		/* they weren't really different */
+
+		/*
+		 * The user file is still unmodified, so just remove it from
+		 * the entry list
+		 */
 		if (!really_quiet)
-		    error (0, 0,
-			   "warning: `%s' is not (any longer) pertinent",
+		    error (0, 0, "`%s' is no longer in the repository",
 			   finfo->fullname);
 		ret = T_REMOVE_ENTRY;
 	    }
@@ -292,7 +298,9 @@ Classify_File (struct file_info *finfo, char *tag, char *date, char *options,
 			error (0, 0, "warning: `%s' was lost", finfo->fullname);
 		ret = T_CHECKOUT;
 	    }
-	    else if (strcmp (vers->ts_user, vers->ts_rcs) == 0)
+	    else if (!strcmp (vers->ts_user,
+			      vers->ts_conflict
+			      ? vers->ts_conflict : vers->ts_rcs))
 	    {
 
 		/*
@@ -306,6 +314,8 @@ Classify_File (struct file_info *finfo, char *tag, char *date, char *options,
 		if (vers->entdata->options &&
 		    strcmp (vers->entdata->options, vers->options) != 0)
 		    ret = T_CHECKOUT;
+		else if (vers->ts_conflict)
+		    ret = T_CONFLICT;
 		else
 		{
 		    sticky_ck (finfo, aflag, vers);
@@ -326,6 +336,13 @@ Classify_File (struct file_info *finfo, char *tag, char *date, char *options,
 		else
 		    ret = T_NEEDS_MERGE;
 #else
+		/* Files with conflict markers and new timestamps fall through
+		 * here, but they need to.  T_CONFLICT is an error in
+		 * commit_fileproc, whereas T_MODIFIED with conflict markers
+		 * is caught but only warned about.  Similarly, update_fileproc
+		 * currently reregisters a file that was conflicted but lost
+		 * its markers.
+		 */
 		ret = T_MODIFIED;
 		sticky_ck (finfo, aflag, vers);
 #endif
