@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1986-2005 The Free Software Foundation, Inc.
+ * Copyright (C) 1986-2006 The Free Software Foundation, Inc.
  *
  * Portions Copyright (C) 1998-2005 Derek Price, Ximbiot <http://ximbiot.com>,
  *                                  and others.
@@ -19,10 +19,23 @@
  *
  */
 
-#include "cvs.h"
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
+
+/* GNULIB headers.  */
 #include "getline.h"
+
+/* CVS headers.  */
 #include "edit.h"
 #include "fileattr.h"
+#include "ignore.h"
+#include "logmsg.h"
+#include "recurse.h"
+#include "repos.h"
+#include "wrapper.h"
+
+#include "cvs.h"
 #include "hardlink.h"
 
 static Dtype check_direntproc (void *callerdat, const char *dir,
@@ -86,7 +99,8 @@ static time_t last_register_time;
 
 static const char *const commit_usage[] =
 {
-    "Usage: %s %s [-cRlf] [-m msg | -F logfile] [-r rev] files...\n",
+    "Usage: %s %s [-cRlf] [-m msg | -F logfile] [-r rev] [files...]\n",
+    "\n",
     "    -c          Check for valid edits before committing.\n",
     "    -R          Process directories recursively.\n",
     "    -l          Local directory only (not recursive).\n",
@@ -332,11 +346,11 @@ copy_ulist (Node *node, void *data)
 
 
 
-#ifdef SERVER_SUPPORT
-# define COMMIT_OPTIONS "+cnlRm:fF:r:"
-#else /* !SERVER_SUPPORT */
-# define COMMIT_OPTIONS "+clRm:fF:r:"
-#endif /* SERVER_SUPPORT */
+/* Commit options both the client and server accept.  */
+#define COMMIT_OPTIONS "+cgG:lRm:fF:r:"
+
+    
+    
 int
 commit (int argc, char **argv)
 {
@@ -344,6 +358,17 @@ commit (int argc, char **argv)
     int err = 0;
     int local = 0;
 
+#ifdef CLIENT_SUPPORT
+    int flags;
+#endif
+
+#ifdef SERVER_SUPPORT
+    /* See below for documentation of the `-n' option.  */
+    const char short_options[] = COMMIT_OPTIONS"n";
+#else /* !SERVER_SUPPORT */
+    const char short_options[] = COMMIT_OPTIONS;
+#endif /* SERVER_SUPPORT */
+
     if (argc == -1)
 	usage (commit_usage);
 
@@ -370,18 +395,20 @@ commit (int argc, char **argv)
 #endif /* CVS_BADROOT */
 
     optind = 0;
-    while ((c = getopt (argc, argv, COMMIT_OPTIONS)) != -1)
+    while ((c = getopt (argc, argv, short_options)) != EOF)
     {
 	switch (c)
 	{
             case 'c':
                 check_valid_edit = 1;
                 break;
+
 #ifdef SERVER_SUPPORT
 	    case 'n':
 		/* Silently ignore -n for compatibility with old
 		 * clients.
 		 */
+		if (!server_active) error(0, 0, "the `-n' option is obsolete");
 		break;
 #endif /* SERVER_SUPPORT */
 	    case 'm':
@@ -599,8 +626,8 @@ commit (int argc, char **argv)
 	   _sure_ why this is needed, but if it is because the "ci"
 	   program, which we used to call, wanted the file to exist,
 	   then it would be relatively simple to fix in the server.  */
-	send_files (find_args.argc, find_args.argv, local, 0,
-		    find_args.force ? SEND_FORCE : 0);
+	flags = find_args.force ? SEND_FORCE : 0;
+	send_files (find_args.argc, find_args.argv, local, 0, flags);
 
 	/* Sending only the names of the files which were modified, added,
 	   or removed means that the server will only do an up-to-date
@@ -629,7 +656,8 @@ commit (int argc, char **argv)
 
 	    fp = cvs_temp_file (&fname);
 	    if (fp == NULL)
-		error (1, 0, "cannot create temporary file %s", fname);
+		error (1, 0, "cannot create temporary file %s",
+		       fname ? fname : "(null)");
 	    if (fwrite (saved_message, 1, strlen (saved_message), fp)
 		!= strlen (saved_message))
 		error (1, errno, "cannot write temporary file %s", fname);
@@ -894,7 +922,8 @@ check_fileproc (void *callerdat, struct file_info *finfo)
 		       finfo->fullname);
 		goto out;
 	    }
-	    if (status == T_MODIFIED && !force_ci && file_has_markers (finfo))
+	    if (status == T_MODIFIED && !force_ci && !really_quiet
+		&& file_has_markers (finfo))
 	    {
 		/* Make this a warning, not an error, because we have
 		   no way of knowing whether the "conflict indicators"
@@ -912,6 +941,22 @@ warning: file `%s' seems to still contain conflict indicators",
 		       finfo->fullname);
 	    }
 
+	    if ((status == T_ADDED || status == T_MODIFIED)
+		&& !force_ci && !really_quiet
+		/* This will not be called from the client.  */
+		&& (get_sign_commits (true) || have_sigfile (finfo->file))
+		&& file_contains_keyword (finfo))
+	    {
+		/* Make this a warning, not an error, because the user may
+		 * be intentionally signing a file with keywords.  Such a file
+		 * may still be verified when checked out -ko.
+		 */
+		if (!quiet)
+		    error (0, 0,
+"warning: signed file `%s' contains at least one RCS keyword",
+			   finfo->fullname);
+	    }
+
 	    if (status == T_REMOVED)
 	    {
 		if (vers->ts_user != NULL)
@@ -1177,6 +1222,28 @@ precommit_list_to_args_proc (p, closure)
 		    arg = p->key;
 		}
 		break;
+            case 'a':
+                li = p->data;
+                switch (li->type)
+                {
+                    case T_ADDED:
+                        arg = "added";
+                        break;
+                    case T_MODIFIED:
+                    case T_CONFLICT:
+                        arg = "modified";
+                        break;
+                    case T_REMOVED:
+                        arg = "removed";
+                        break;
+                    case T_TITLE:
+                        arg = "imported";
+                        break;
+		    default:
+			error (1, 0, "Unexpected action type %d.", li->type);
+                        break;
+                }
+                break;
 	    default:
 		error (1, 0,
 		       "Unknown format character or not a list attribute: %c",
@@ -1255,7 +1322,7 @@ precommit_proc (const char *repository, const char *filter, void *closure)
 #endif /* SERVER_SUPPORT */
 			      "p", "s", srepos,
 			      "r", "s", current_parsed_root->directory,
-			      "s", ",", ulist, precommit_list_to_args_proc,
+			      "sa", ",", ulist, precommit_list_to_args_proc,
 			      (void *) NULL,
 			      (char *) NULL);
 
@@ -1513,9 +1580,15 @@ out:
            will return the version number of a file even after it has
            been removed from the archive, which is not the behavior we
            want for our commitlog messages; we want the old version
-           number and then "NONE." */
-
-	if (ci->status != T_REMOVED)
+           number and then "NONE, unless UseNewInfoFmtStrings has
+           been specified in the config.  Expose the real version in that
+           case and allow the trigger scripts to decide how to use it.  */
+            
+        if (ci->status != T_REMOVED
+#ifdef SUPPORT_OLD_INFO_FMT_STRINGS
+	    || config->UseNewInfoFmtStrings
+#endif
+	    )
 	{
 	    p = findnode (ulist, finfo->file);
 	    if (p)
@@ -1825,11 +1898,15 @@ remove_file (struct file_info *finfo, char *tag, char *message)
 	if (!quiet)
 	    error (0, retcode == -1 ? errno : 0,
 		   "failed to commit dead revision for `%s'", finfo->fullname);
+	if (prev_rev != NULL)
+	    free (prev_rev);
 	return 1;
     }
     /* At this point, the file has been committed as removed.  We should
        probably tell the history file about it  */
-    history_write ('R', NULL, finfo->rcs->head, finfo->file, finfo->repository);
+    corev = rev ? RCS_getbranch (finfo->rcs, rev, 1) : RCS_head (finfo->rcs);
+    history_write ('R', NULL, corev, finfo->file, finfo->repository);
+    free (corev);
 
     if (rev != NULL)
 	free (rev);
@@ -2096,7 +2173,9 @@ checkaddfile (const char *file, const char *repository, const char *tag,
 	    /* We tell the user about this, because it means that the
 	       old revisions will no longer retrieve the way that they
 	       used to.  */
-	    error (0, 0, "changing keyword expansion mode to %s", options);
+	    error (0, 0,
+"changing keyword expansion mode of `%s' from `-k%s' to `%s'",
+		   file, oldexpand ? oldexpand : "kv", options);
 	    RCS_setexpand (rcs, options + 2);
 	}
 
@@ -2126,7 +2205,7 @@ checkaddfile (const char *file, const char *repository, const char *tag,
 	if (lock_RCS (file, rcs, rev, repository))
 	{
 	    error (0, 0, "cannot lock revision %s in `%s'.",
-		   rev ? rev : tag ? tag : "HEAD", rcs->path);
+		   rev ? rev : tag ? tag : TAG_HEAD, rcs->path);
 	    if (rev != NULL)
 		free (rev);
 	    goto out;
@@ -2181,7 +2260,7 @@ checkaddfile (const char *file, const char *repository, const char *tag,
 	    rcs = RCS_parse (file, repository);
 	    if (rcs == NULL)
 	    {
-		error (0, 0, "could not read %s", rcs->path);
+		error (0, 0, "could not read %s in %s", file, repository);
 		goto out;
 	    }
 	    *rcsnode = rcs;
