@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1986-2005 The Free Software Foundation, Inc.
+ * Copyright (C) 1986-2006 The Free Software Foundation, Inc.
  *
  * Portions Copyright (C) 1998-2005 Derek Price, Ximbiot <http://ximbiot.com>,
  *                                  and others.
@@ -19,11 +19,28 @@
  * files.
  */
 
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
+
+/* Verify interface.  */
+#include "diff.h"
+
+/* CVS headers.  */
+#include "base.h"
+#include "ignore.h"
+#include "rcs.h"
+#include "recurse.h"
+#include "wrapper.h"
+
 #include "cvs.h"
 
+
+
 enum diff_file
 {
     DIFF_ERROR,
+    DIFF_CLIENT,
     DIFF_ADDED,
     DIFF_REMOVED,
     DIFF_DIFFERENT,
@@ -39,10 +56,7 @@ static int diff_filesdoneproc (void *callerdat, int err,
 static int diff_dirleaveproc (void *callerdat, const char *dir,
                               int err, const char *update_dir,
                               List *entries);
-static enum diff_file diff_file_nodiff (struct file_info *finfo, Vers_TS *vers,
-                                        enum diff_file, char **rev1_cache );
 static int diff_fileproc (void *callerdat, struct file_info *finfo);
-static void diff_mark_errors (int err);
 
 
 /* Global variables.  Would be cleaner if we just put this stuff in a
@@ -52,8 +66,7 @@ static void diff_mark_errors (int err);
 static char *diff_rev1, *diff_rev2;
 /* Command line dates, from -D option.  Malloc'd.  */
 static char *diff_date1, *diff_date2;
-static char *use_rev1, *use_rev2;
-static int have_rev1_label, have_rev2_label;
+static bool have_rev1_label, have_rev2_label;
 
 /* Revision of the user file, if it is unchanged from something in the
    repository and we want to use that fact.  */
@@ -64,7 +77,7 @@ static char **diff_argv;
 static int diff_argc;
 static size_t diff_arg_allocated;
 static int diff_errors;
-static int empty_files;
+static bool empty_files;
 
 static const char *const diff_usage[] =
 {
@@ -290,8 +303,6 @@ diff (int argc, char **argv)
     if (argc == -1)
 	usage (diff_usage);
 
-    have_rev1_label = have_rev2_label = 0;
-
     /*
      * Note that we catch all the valid arguments here, so that we can
      * intercept the -r arguments for doing revision diffs; and -l/-R for a
@@ -389,7 +400,7 @@ diff (int argc, char **argv)
 		    diff_date1 = Make_Date (optarg);
 		break;
 	    case 'N':
-		empty_files = 1;
+		empty_files = true;
 		break;
 	    case '?':
 	    default:
@@ -405,7 +416,10 @@ diff (int argc, char **argv)
 	options = xstrdup ("");
 
 #ifdef CLIENT_SUPPORT
-    if (current_parsed_root->isremote) {
+    if (current_parsed_root->isremote)
+    {
+	int flags;
+
 	/* We're the client side.  Fire up the remote server.  */
 	start_server ();
 	
@@ -429,10 +443,12 @@ diff (int argc, char **argv)
 	send_arg ("--");
 
 	/* Send the current files unless diffing two revs from the archive */
-	if (!diff_rev2 && !diff_date2)
-	    send_files (argc, argv, local, 0, 0);
-	else
-	    send_files (argc, argv, local, 0, SEND_NO_CONTENTS);
+	flags = 0;
+	if ((!suppress_bases && supported_request ("Base-diff"))
+	    || diff_rev2 || diff_date2)
+	    flags |= SEND_NO_CONTENTS;
+
+	send_files (argc, argv, local, 0, flags);
 
 	send_file_names (argc, argv, SEND_EXPAND_WILD);
 
@@ -440,7 +456,7 @@ diff (int argc, char **argv)
         err = get_responses_and_close ();
 	free (options);
 	options = NULL;
-	return err;
+	return diff_errors ? diff_errors : err;
     }
 #endif
 
@@ -474,6 +490,259 @@ diff (int argc, char **argv)
 
 
 
+/*
+ * verify that a file is different
+ *
+ * INPUTS
+ *   finfo
+ *   vers
+ *   empty_file
+ *
+ * OUTPUTS
+ *   rev1_cache		Cache the contents of rev1 if we look it up.
+ */
+static enum diff_file
+diff_file_nodiff (struct file_info *finfo, Vers_TS *vers,
+                  enum diff_file empty_file, char **rev1_cache,
+		  const char *arg_rev1, char **use_rev1, char **use_rev2)
+{
+    Vers_TS *xvers;
+    int retcode;
+
+    TRACE (TRACE_FUNCTION, "diff_file_nodiff (%s, %d)",
+           finfo->fullname ? finfo->fullname : "(null)", empty_file);
+
+    /* free up any old use_rev* variables and reset 'em */
+    *use_rev1 = *use_rev2 = NULL;
+
+    if (diff_rev1 || diff_date1)
+    {
+	/* special handling for TAG_HEAD */
+	if (diff_rev1 && strcmp (diff_rev1, TAG_HEAD) == 0)
+	{
+	    if (vers->vn_rcs != NULL && vers->srcfile != NULL)
+		*use_rev1 = RCS_branch_head (vers->srcfile, vers->vn_rcs);
+	}
+	else
+	{
+	    xvers = Version_TS (finfo, NULL, diff_rev1, diff_date1, 1, 0);
+	    if (xvers->vn_rcs != NULL)
+		*use_rev1 = xstrdup (xvers->vn_rcs);
+	    freevers_ts (&xvers);
+	}
+    }
+    if (diff_rev2 || diff_date2)
+    {
+	/* special handling for TAG_HEAD */
+	if (diff_rev2 && strcmp (diff_rev2, TAG_HEAD) == 0)
+	{
+	    if (vers->vn_rcs && vers->srcfile)
+		*use_rev2 = RCS_branch_head (vers->srcfile, vers->vn_rcs);
+	}
+	else
+	{
+	    xvers = Version_TS (finfo, NULL, diff_rev2, diff_date2, 1, 0);
+	    if (xvers->vn_rcs != NULL)
+		*use_rev2 = xstrdup (xvers->vn_rcs);
+	    freevers_ts (&xvers);
+	}
+
+	if (!*use_rev1 || RCS_isdead (vers->srcfile, *use_rev1))
+	{
+	    /* The first revision does not exist.  If EMPTY_FILES is
+               true, treat this as an added file.  Otherwise, warn
+               about the missing tag.  */
+	    if (!*use_rev2 || RCS_isdead (vers->srcfile, *use_rev2))
+		/* At least in the case where DIFF_REV1 and DIFF_REV2
+		 * are both numeric (and non-existant (NULL), as opposed to
+		 * dead?), we should be returning some kind of error (see
+		 * basicb-8a0 in testsuite).  The symbolic case may be more
+		 * complicated.
+		 */
+		return DIFF_SAME;
+	    if (empty_files)
+		return DIFF_ADDED;
+	    if (*use_rev1)
+	    {
+		if (diff_rev1)
+		{
+		    error (0, 0,
+		       "Tag %s refers to a dead (removed) revision in file `%s'.",
+		       diff_rev1, finfo->fullname);
+		}
+		else
+		{
+		    error (0, 0,
+		       "Date %s refers to a dead (removed) revision in file `%s'.",
+		       diff_date1, finfo->fullname);
+		}
+		error (0, 0,
+		       "No comparison available.  Pass `-N' to `%s diff'?",
+		       program_name);
+	    }
+	    else if (diff_rev1)
+		error (0, 0, "tag %s is not in file %s", diff_rev1,
+		       finfo->fullname);
+	    else
+		error (0, 0, "no revision for date %s in file %s",
+		       diff_date1, finfo->fullname);
+	    return DIFF_ERROR;
+	}
+
+	assert (*use_rev1);
+	if (!*use_rev2 || RCS_isdead (vers->srcfile, *use_rev2))
+	{
+	    /* The second revision does not exist.  If EMPTY_FILES is
+               true, treat this as a removed file.  Otherwise warn
+               about the missing tag.  */
+	    if (empty_files)
+		return DIFF_REMOVED;
+	    if (*use_rev2)
+	    {
+		if (diff_rev2)
+		{
+		    error( 0, 0,
+		       "Tag %s refers to a dead (removed) revision in file `%s'.",
+		       diff_rev2, finfo->fullname );
+		}
+		else
+		{
+		    error( 0, 0,
+		       "Date %s refers to a dead (removed) revision in file `%s'.",
+		       diff_date2, finfo->fullname );
+		}
+		error( 0, 0,
+		       "No comparison available.  Pass `-N' to `%s diff'?",
+		       program_name );
+	    }
+	    else if (diff_rev2)
+		error (0, 0, "tag %s is not in file %s", diff_rev2,
+		       finfo->fullname);
+	    else
+		error (0, 0, "no revision for date %s in file %s",
+		       diff_date2, finfo->fullname);
+	    return DIFF_ERROR;
+	}
+	/* Now, see if we really need to do the diff.  We can't assume that the
+	 * files are different when the revs are.
+	 */
+	assert (*use_rev2);
+	if (!strcmp (*use_rev1, *use_rev2))
+	    return DIFF_SAME;
+	/* else fall through and do the diff */
+    }
+
+    /* If we had a r1/d1 & r2/d2, then at this point we must have a C3P0...
+     * err...  ok, then both rev1 & rev2 must have resolved to an existing,
+     * live version due to if statement we just closed.
+     */
+    assert (!(diff_rev2 || diff_date2) || (*use_rev1 && *use_rev2));
+
+    if ((diff_rev1 || diff_date1) &&
+	(!*use_rev1 || RCS_isdead (vers->srcfile, *use_rev1)))
+    {
+	/* The first revision does not exist, and no second revision
+           was given.  */
+	if (empty_files)
+	{
+	    if (empty_file == DIFF_REMOVED)
+		return DIFF_SAME;
+	    if (user_file_rev && !*use_rev2)
+		*use_rev2 = xstrdup (user_file_rev);
+	    return DIFF_ADDED;
+	}
+	if (*use_rev1)
+	{
+	    if (diff_rev1)
+	    {
+		error( 0, 0,
+		   "Tag %s refers to a dead (removed) revision in file `%s'.",
+		   diff_rev1, finfo->fullname );
+	    }
+	    else
+	    {
+		error( 0, 0,
+		   "Date %s refers to a dead (removed) revision in file `%s'.",
+		   diff_date1, finfo->fullname );
+	    }
+	    error( 0, 0,
+		   "No comparison available.  Pass `-N' to `%s diff'?",
+		   program_name );
+	}
+	else if ( diff_rev1 )
+	    error( 0, 0, "tag %s is not in file %s", diff_rev1,
+		   finfo->fullname );
+	else
+	    error( 0, 0, "no revision for date %s in file %s",
+		   diff_date1, finfo->fullname );
+	return DIFF_ERROR;
+    }
+
+    assert (!diff_rev1 || *use_rev1);
+
+    if (user_file_rev)
+    {
+        /* drop user_file_rev into first unused use_rev */
+        if (!*use_rev1) 
+	    *use_rev1 = xstrdup (user_file_rev);
+	else if (!*use_rev2)
+	    *use_rev2 = xstrdup (user_file_rev);
+	/* and if not, it wasn't needed anyhow */
+	user_file_rev = NULL;
+    }
+
+    /* Now, see if we really need to do the diff.  We can't assume that the
+     * files are different when the revs are.
+     */
+    if (*use_rev1 && *use_rev2) 
+    {
+	if (!strcmp (*use_rev1, *use_rev2))
+	    return DIFF_SAME;
+	/* Fall through and do the diff. */
+    }
+    /* Don't want to do the timestamp check with both *USE_REV1 & *USE_REV2 set.
+     * The timestamp check is just for the default case of diffing the
+     * workspace file against its base revision.
+     */
+    else if (!*use_rev1
+             || (vers->vn_user
+                 && !strcmp (*use_rev1, vers->vn_user)))
+    {
+	if ((empty_file == DIFF_DIFFERENT || empty_file == DIFF_CLIENT)
+	    && vers->ts_user != NULL
+	    && strcmp (vers->ts_rcs, vers->ts_user) == 0
+	    && (!(*options) || strcmp (options, vers->options) == 0))
+	{
+	    return DIFF_SAME;
+	}
+	if (!*use_rev1
+	    && (vers->vn_user[0] != '0' || vers->vn_user[1] != '\0'))
+	{
+	    if (vers->vn_user[0] == '-')
+		*use_rev1 = xstrdup (vers->vn_user + 1);
+	    else
+		*use_rev1 = xstrdup (vers->vn_user);
+	}
+    }
+
+    /* If we already know that the file is being added or removed,
+       then we don't want to do an actual file comparison here.  */
+    if (empty_file != DIFF_DIFFERENT)
+	return empty_file;
+
+    /*
+     * Run a quick cmp to see if we should bother with a full diff.
+     */
+
+    retcode = RCS_cmp_file (vers->srcfile, diff_rev1, *use_rev1, rev1_cache,
+                            *use_rev2, *options ? options : vers->options,
+			    finfo->file);
+
+    return retcode == 0 ? DIFF_SAME : DIFF_DIFFERENT;
+}
+
+
+
 /*
  * Do a file diff
  */
@@ -481,15 +750,14 @@ diff (int argc, char **argv)
 static int
 diff_fileproc (void *callerdat, struct file_info *finfo)
 {
-    int status, err = 2;		/* 2 == trouble, like rcsdiff */
+    int err = 2;		/* 2 == trouble, like rcsdiff */
     Vers_TS *vers;
-    enum diff_file empty_file = DIFF_DIFFERENT;
-    char *tmp = NULL;
-    char *tocvsPath = NULL;
-    char *fname = NULL;
-    char *label1;
-    char *label2;
+    enum diff_file empty_file = server_use_bases ()
+				? DIFF_CLIENT : DIFF_DIFFERENT;
     char *rev1_cache = NULL;
+    char *use_rev1 = NULL, *use_rev2 = NULL;
+    const char *f1 = NULL, *f2 = NULL;
+    char *label1 = NULL, *label2 = NULL;
 
     user_file_rev = 0;
     vers = Version_TS (finfo, NULL, NULL, NULL, 1, 0);
@@ -639,7 +907,10 @@ diff_fileproc (void *callerdat, struct file_info *finfo)
 	}
     }
 
-    empty_file = diff_file_nodiff (finfo, vers, empty_file, &rev1_cache);
+    empty_file = diff_file_nodiff (finfo, vers, empty_file, &rev1_cache,
+				   diff_rev1 && !isdigit (diff_rev1[0])
+				   ? diff_rev1 : vers->tag,
+				   &use_rev1, &use_rev2);
     if (empty_file == DIFF_SAME)
     {
 	/* In the server case, would be nice to send a "Checked-in"
@@ -655,150 +926,102 @@ diff_fileproc (void *callerdat, struct file_info *finfo)
     else if (empty_file == DIFF_ERROR)
 	goto out;
 
-    /* Output an "Index:" line for patch to use */
-    cvs_output ("Index: ", 0);
-    cvs_output (finfo->fullname, 0);
-    cvs_output ("\n", 1);
-
-    tocvsPath = wrap_tocvs_process_file (finfo->file);
-    if (tocvsPath)
+    if (empty_file == DIFF_ADDED)
+	f1 = DEVNULL;
+    else if (rev1_cache)
     {
-	/* Backup the current version of the file to CVS/,,filename */
-	fname = Xasprintf (fname, "%s/%s%s", CVSADM, CVSPREFIX, finfo->file);
-	if (unlink_file_dir (fname) < 0)
-	    if (!existence_error (errno))
-		error (1, errno, "cannot remove %s", fname);
-	rename_file (finfo->file, fname);
-	/* Copy the wrapped file to the current directory then go to work */
-	copy_file (tocvsPath, finfo->file);
+	/* If this is cached, temp_checkout was not called for the client.
+	 */
+	assert (empty_file != DIFF_CLIENT);
+	f1 = rev1_cache;
     }
+    else
+    {
+	f1 = temp_checkout (vers->srcfile, finfo,
+			    vers->vn_user && *(vers->vn_user) == '-'
+			    ? vers->vn_user + 1 : vers->vn_user,
+			    use_rev1, vers->tag,
+			    diff_rev1 && !isdigit (diff_rev1[0])
+			    ? diff_rev1 : vers->tag,
+			    vers->options, *options ? options : vers->options);
+	if (!f1)
+	    goto out;
+    }
+
+    if (empty_file == DIFF_REMOVED)
+	f2 = DEVNULL;
+    else if (use_rev2)
+    {
+	f2 = temp_checkout (vers->srcfile, finfo,
+			    vers->vn_user && *(vers->vn_user) == '-'
+			    ? vers->vn_user + 1 : vers->vn_user,
+			    use_rev2, vers->tag,
+			    diff_rev2 && !isdigit (diff_rev2[0])
+			    ? diff_rev2 : NULL,
+			    vers->options, *options ? options : vers->options);
+	if (!f2)
+	    goto out;
+    }
+    else
+	f2 = finfo->file;
 
     /* Set up file labels appropriate for compatibility with the Larry Wall
      * implementation of patch if the user didn't specify.  This is irrelevant
      * according to the POSIX.2 specification.
-     */
-    label1 = NULL;
-    label2 = NULL;
-    /* The user cannot set the rev2 label without first setting the rev1
+     *
+     * The user cannot set the rev2 label without first setting the rev1
      * label.
      */
     if (!have_rev2_label)
     {
 	if (empty_file == DIFF_REMOVED)
-	    label2 = make_file_label (DEVNULL, NULL, NULL);
+	    label2 = make_file_label (DEVNULL, NULL, NULL, NULL);
 	else
 	    label2 = make_file_label (finfo->fullname, use_rev2,
-	                              vers->srcfile);
+				      /* FIXME: make_file_label should be able
+				       * to trust vers->ts_user, but
+				       * Version_TS isn't setting it correctly
+				       * when base files are suppressed.
+				       */
+	                              server_active
+				      ? "Is-modified" : vers->ts_user,
+				      finfo->rcs);
 	if (!have_rev1_label)
 	{
 	    if (empty_file == DIFF_ADDED)
-		label1 = make_file_label (DEVNULL, NULL, NULL);
+		label1 = make_file_label (DEVNULL, NULL, NULL, NULL);
 	    else
 		label1 = make_file_label (finfo->fullname, use_rev1,
-		                          vers->srcfile);
+		                          NULL, finfo->rcs);
 	}
     }
 
-    if (empty_file == DIFF_ADDED || empty_file == DIFF_REMOVED)
-    {
-	/* This is fullname, not file, possibly despite the POSIX.2
-	 * specification, because that's the way all the Larry Wall
-	 * implementations of patch (are there other implementations?) want
-	 * things and the POSIX.2 spec appears to leave room for this.
-	 */
-	cvs_output ("\
-===================================================================\n\
-RCS file: ", 0);
-	cvs_output (finfo->fullname, 0);
-	cvs_output ("\n", 1);
-
-	cvs_output ("diff -N ", 0);
-	cvs_output (finfo->fullname, 0);
-	cvs_output ("\n", 1);
-
-	if (empty_file == DIFF_ADDED)
-	{
-	    if (use_rev2 == NULL)
-                status = diff_exec (DEVNULL, finfo->file, label1, label2,
-				    diff_argc, diff_argv, RUN_TTY);
-	    else
-	    {
-		int retcode;
-
-		tmp = cvs_temp_name ();
-		retcode = RCS_checkout (vers->srcfile, NULL, use_rev2, NULL,
-					*options ? options : vers->options,
-					tmp, NULL, NULL);
-		if (retcode != 0)
-		    goto out;
-
-		status = diff_exec (DEVNULL, tmp, label1, label2,
-				    diff_argc, diff_argv, RUN_TTY);
-	    }
-	}
-	else
-	{
-	    int retcode;
-
-	    tmp = cvs_temp_name ();
-	    retcode = RCS_checkout (vers->srcfile, NULL, use_rev1, NULL,
-				    *options ? options : vers->options,
-				    tmp, NULL, NULL);
-	    if (retcode != 0)
-		goto out;
-
-	    status = diff_exec (tmp, DEVNULL, label1, label2,
-				diff_argc, diff_argv, RUN_TTY);
-	}
-    }
-    else
-    {
-	status = RCS_exec_rcsdiff (vers->srcfile, diff_argc, diff_argv,
-                                   *options ? options : vers->options,
-                                   use_rev1, rev1_cache, use_rev2,
-                                   label1, label2, finfo->file);
-
-    }
-
-    if (label1) free (label1);
-    if (label2) free (label2);
-
-    switch (status)
-    {
-	case -1:			/* fork failed */
-	    error (1, errno, "fork failed while diffing %s",
-		   vers->srcfile->path);
-	case 0:				/* everything ok */
-	    err = 0;
-	    break;
-	default:			/* other error */
-	    err = status;
-	    break;
-    }
+    err = base_diff (finfo, diff_argc, diff_argv, f1, use_rev1, label1,
+		     f2, use_rev2, label2, empty_files);
 
 out:
-    if( tocvsPath != NULL )
+    if (empty_file != DIFF_ADDED && !rev1_cache && f1)
     {
-	if (unlink_file_dir (finfo->file) < 0)
-	    if (! existence_error (errno))
-		error (1, errno, "cannot remove %s", finfo->file);
-
-	rename_file (fname, finfo->file);
-	if (unlink_file (tocvsPath) < 0)
-	    error (1, errno, "cannot remove %s", tocvsPath);
-	free (fname);
+	if (unlink_file (f1) < 0)
+	    error (0, errno, "Failed to remove temp file `%s'", f1);
+	free ((char *)f1);
     }
+    if (empty_file != DIFF_REMOVED && use_rev2 && f2)
+    {
+	if (unlink_file (f2) < 0)
+	    error (0, errno, "Failed to remove temp file `%s'", f2);
+	free ((char *)f2);
+    }
+
+    if (use_rev1) free (use_rev1);
+    if (use_rev2) free (use_rev2);
+    if (label1) free (label1);
+    if (label2) free (label2);
 
     /* Call CVS_UNLINK() rather than unlink_file() below to avoid the check
      * for noexec.
      */
-    if (tmp != NULL)
-    {
-	if (CVS_UNLINK (tmp) < 0)
-	    error (0, errno, "cannot remove %s", tmp);
-	free (tmp);
-    }
-    if (rev1_cache != NULL)
+    if (rev1_cache)
     {
 	if (CVS_UNLINK (rev1_cache) < 0)
 	    error (0, errno, "cannot remove %s", rev1_cache);
@@ -807,6 +1030,7 @@ out:
 
     freevers_ts (&vers);
     diff_mark_errors (err);
+
     return err;
 }
 
@@ -815,7 +1039,7 @@ out:
 /*
  * Remember the exit status for each file.
  */
-static void
+void
 diff_mark_errors (int err)
 {
     if (err > diff_errors)
@@ -873,256 +1097,17 @@ diff_dirleaveproc (void *callerdat, const char *dir, int err,
 
 
 
-/*
- * verify that a file is different
- *
- * INPUTS
- *   finfo
- *   vers
- *   empty_file
- *
- * OUTPUTS
- *   rev1_cache		Cache the contents of rev1 if we look it up.
- */
-static enum diff_file
-diff_file_nodiff (struct file_info *finfo, Vers_TS *vers,
-                  enum diff_file empty_file, char **rev1_cache)
+struct diff_info *
+get_diff_info (void)
 {
-    Vers_TS *xvers;
-    int retcode;
+    static struct diff_info di;
 
-    TRACE (TRACE_FUNCTION, "diff_file_nodiff (%s, %d)",
-           finfo->fullname ? finfo->fullname : "(null)", empty_file);
+    if (strcmp ("diff", cvs_cmd_name))
+	return NULL;
 
-    /* free up any old use_rev* variables and reset 'em */
-    if (use_rev1)
-	free (use_rev1);
-    if (use_rev2)
-	free (use_rev2);
-    use_rev1 = use_rev2 = NULL;
+    di.diff_argc = diff_argc;
+    di.diff_argv = diff_argv;
+    di.empty_files = empty_files;
 
-    if (diff_rev1 || diff_date1)
-    {
-	/* special handling for TAG_HEAD */
-	if (diff_rev1 && strcmp (diff_rev1, TAG_HEAD) == 0)
-	{
-	    if (vers->vn_rcs != NULL && vers->srcfile != NULL)
-		use_rev1 = RCS_branch_head (vers->srcfile, vers->vn_rcs);
-	}
-	else
-	{
-	    xvers = Version_TS (finfo, NULL, diff_rev1, diff_date1, 1, 0);
-	    if (xvers->vn_rcs != NULL)
-		use_rev1 = xstrdup (xvers->vn_rcs);
-	    freevers_ts (&xvers);
-	}
-    }
-    if (diff_rev2 || diff_date2)
-    {
-	/* special handling for TAG_HEAD */
-	if (diff_rev2 && strcmp (diff_rev2, TAG_HEAD) == 0)
-	{
-	    if (vers->vn_rcs && vers->srcfile)
-		use_rev2 = RCS_branch_head (vers->srcfile, vers->vn_rcs);
-	}
-	else
-	{
-	    xvers = Version_TS (finfo, NULL, diff_rev2, diff_date2, 1, 0);
-	    if (xvers->vn_rcs != NULL)
-		use_rev2 = xstrdup (xvers->vn_rcs);
-	    freevers_ts (&xvers);
-	}
-
-	if (use_rev1 == NULL || RCS_isdead (vers->srcfile, use_rev1))
-	{
-	    /* The first revision does not exist.  If EMPTY_FILES is
-               true, treat this as an added file.  Otherwise, warn
-               about the missing tag.  */
-	    if (use_rev2 == NULL || RCS_isdead (vers->srcfile, use_rev2))
-		/* At least in the case where DIFF_REV1 and DIFF_REV2
-		 * are both numeric (and non-existant (NULL), as opposed to
-		 * dead?), we should be returning some kind of error (see
-		 * basicb-8a0 in testsuite).  The symbolic case may be more
-		 * complicated.
-		 */
-		return DIFF_SAME;
-	    if (empty_files)
-		return DIFF_ADDED;
-	    if (use_rev1 != NULL)
-	    {
-		if (diff_rev1)
-		{
-		    error (0, 0,
-		       "Tag %s refers to a dead (removed) revision in file `%s'.",
-		       diff_rev1, finfo->fullname);
-		}
-		else
-		{
-		    error (0, 0,
-		       "Date %s refers to a dead (removed) revision in file `%s'.",
-		       diff_date1, finfo->fullname);
-		}
-		error (0, 0,
-		       "No comparison available.  Pass `-N' to `%s diff'?",
-		       program_name);
-	    }
-	    else if (diff_rev1)
-		error (0, 0, "tag %s is not in file %s", diff_rev1,
-		       finfo->fullname);
-	    else
-		error (0, 0, "no revision for date %s in file %s",
-		       diff_date1, finfo->fullname);
-	    return DIFF_ERROR;
-	}
-
-	assert( use_rev1 != NULL );
-	if( use_rev2 == NULL || RCS_isdead( vers->srcfile, use_rev2 ) )
-	{
-	    /* The second revision does not exist.  If EMPTY_FILES is
-               true, treat this as a removed file.  Otherwise warn
-               about the missing tag.  */
-	    if (empty_files)
-		return DIFF_REMOVED;
-	    if( use_rev2 != NULL )
-	    {
-		if (diff_rev2)
-		{
-		    error( 0, 0,
-		       "Tag %s refers to a dead (removed) revision in file `%s'.",
-		       diff_rev2, finfo->fullname );
-		}
-		else
-		{
-		    error( 0, 0,
-		       "Date %s refers to a dead (removed) revision in file `%s'.",
-		       diff_date2, finfo->fullname );
-		}
-		error( 0, 0,
-		       "No comparison available.  Pass `-N' to `%s diff'?",
-		       program_name );
-	    }
-	    else if (diff_rev2)
-		error (0, 0, "tag %s is not in file %s", diff_rev2,
-		       finfo->fullname);
-	    else
-		error (0, 0, "no revision for date %s in file %s",
-		       diff_date2, finfo->fullname);
-	    return DIFF_ERROR;
-	}
-	/* Now, see if we really need to do the diff.  We can't assume that the
-	 * files are different when the revs are.
-	 */
-	assert( use_rev2 != NULL );
-	if( strcmp (use_rev1, use_rev2) == 0 )
-	    return DIFF_SAME;
-	/* else fall through and do the diff */
-    }
-
-    /* If we had a r1/d1 & r2/d2, then at this point we must have a C3P0...
-     * err...  ok, then both rev1 & rev2 must have resolved to an existing,
-     * live version due to if statement we just closed.
-     */
-    assert (!(diff_rev2 || diff_date2) || (use_rev1 && use_rev2));
-
-    if ((diff_rev1 || diff_date1) &&
-	(use_rev1 == NULL || RCS_isdead (vers->srcfile, use_rev1)))
-    {
-	/* The first revision does not exist, and no second revision
-           was given.  */
-	if (empty_files)
-	{
-	    if (empty_file == DIFF_REMOVED)
-		return DIFF_SAME;
-	    if( user_file_rev && use_rev2 == NULL )
-		use_rev2 = xstrdup( user_file_rev );
-	    return DIFF_ADDED;
-	}
-	if( use_rev1 != NULL )
-	{
-	    if (diff_rev1)
-	    {
-		error( 0, 0,
-		   "Tag %s refers to a dead (removed) revision in file `%s'.",
-		   diff_rev1, finfo->fullname );
-	    }
-	    else
-	    {
-		error( 0, 0,
-		   "Date %s refers to a dead (removed) revision in file `%s'.",
-		   diff_date1, finfo->fullname );
-	    }
-	    error( 0, 0,
-		   "No comparison available.  Pass `-N' to `%s diff'?",
-		   program_name );
-	}
-	else if ( diff_rev1 )
-	    error( 0, 0, "tag %s is not in file %s", diff_rev1,
-		   finfo->fullname );
-	else
-	    error( 0, 0, "no revision for date %s in file %s",
-		   diff_date1, finfo->fullname );
-	return DIFF_ERROR;
-    }
-
-    assert( !diff_rev1 || use_rev1 );
-
-    if (user_file_rev)
-    {
-        /* drop user_file_rev into first unused use_rev */
-        if (!use_rev1) 
-	    use_rev1 = xstrdup (user_file_rev);
-	else if (!use_rev2)
-	    use_rev2 = xstrdup (user_file_rev);
-	/* and if not, it wasn't needed anyhow */
-	user_file_rev = NULL;
-    }
-
-    /* Now, see if we really need to do the diff.  We can't assume that the
-     * files are different when the revs are.
-     */
-    if( use_rev1 && use_rev2) 
-    {
-	if (strcmp (use_rev1, use_rev2) == 0)
-	    return DIFF_SAME;
-	/* Fall through and do the diff. */
-    }
-    /* Don't want to do the timestamp check with both use_rev1 & use_rev2 set.
-     * The timestamp check is just for the default case of diffing the
-     * workspace file against its base revision.
-     */
-    else if( use_rev1 == NULL
-             || ( vers->vn_user != NULL
-                  && strcmp( use_rev1, vers->vn_user ) == 0 ) )
-    {
-	if (empty_file == DIFF_DIFFERENT
-	    && vers->ts_user != NULL
-	    && strcmp (vers->ts_rcs, vers->ts_user) == 0
-	    && (!(*options) || strcmp (options, vers->options) == 0))
-	{
-	    return DIFF_SAME;
-	}
-	if (use_rev1 == NULL
-	    && (vers->vn_user[0] != '0' || vers->vn_user[1] != '\0'))
-	{
-	    if (vers->vn_user[0] == '-')
-		use_rev1 = xstrdup (vers->vn_user + 1);
-	    else
-		use_rev1 = xstrdup (vers->vn_user);
-	}
-    }
-
-    /* If we already know that the file is being added or removed,
-       then we don't want to do an actual file comparison here.  */
-    if (empty_file != DIFF_DIFFERENT)
-	return empty_file;
-
-    /*
-     * Run a quick cmp to see if we should bother with a full diff.
-     */
-
-    retcode = RCS_cmp_file( vers->srcfile, use_rev1, rev1_cache,
-                            use_rev2, *options ? options : vers->options,
-			    finfo->file );
-
-    return retcode == 0 ? DIFF_SAME : DIFF_DIFFERENT;
+    return &di;
 }
