@@ -1,23 +1,41 @@
-/* Implementation for "cvs edit", "cvs watch on", and related commands
+/*
+ * Copyright (C) 2006 The Free Software Foundation, Inc.
+ *
+ * Implementation for "cvs edit", "cvs watch on", and related commands
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2, or (at your option)
+ * any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ */
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.  */
+/* Verify interface.  */
+#include "edit.h"
 
-#include "cvs.h"
+/* GNULIB headers.  */
 #include "getline.h"
 #include "yesno.h"
+
+/* CVS headers.  */
+#include "base.h"
+#include "ignore.h"
+#include "recurse.h"
+#include "repos.h"
+
+#include "cvs.h"
 #include "watch.h"
-#include "edit.h"
 #include "fileattr.h"
 
-static int watch_onoff (int, char **);
+
 
 static bool check_edited = false;
 static int setting_default;
@@ -32,7 +50,10 @@ static bool setting_tcommit;
 static int
 onoff_fileproc (void *callerdat, struct file_info *finfo)
 {
+    char *watched = fileattr_get0 (finfo->file, "_watched");
     fileattr_set (finfo->file, "_watched", turning_on ? "" : NULL);
+    if (watched != NULL)
+	free (watched);
     return 0;
 }
 
@@ -43,7 +64,12 @@ onoff_filesdoneproc (void *callerdat, int err, const char *repository,
                      const char *update_dir, List *entries)
 {
     if (setting_default)
+    {
+	char *watched = fileattr_get0 (NULL, "_watched");
 	fileattr_set (NULL, "_watched", turning_on ? "" : NULL);
+	if (watched != NULL)
+	    free (watched);
+    }
     return err;
 }
 
@@ -121,6 +147,7 @@ watch_off (int argc, char **argv)
 
 
 
+#ifdef CLIENT_SUPPORT
 static int
 dummy_fileproc (void *callerdat, struct file_info *finfo)
 {
@@ -129,6 +156,7 @@ dummy_fileproc (void *callerdat, struct file_info *finfo)
        pass NULL for fileproc.  */
     return 0;
 }
+#endif
 
 
 
@@ -316,24 +344,49 @@ edit_file (void *data, List *ent_list, const char *short_pathname,
 {
     Node *node;
     struct file_info finfo;
-    char *basefilename;
+    char *editbasefn, *basefn, *rev;
 
-    xchmod (filename, 1);
-
-    mkdir_if_needed (CVSADM_BASE);
-    basefilename = Xasprintf ("%s/%s", CVSADM_BASE, filename);
-    copy_file (filename, basefilename);
-    free (basefilename);
 
     node = findnode_fn (ent_list, filename);
-    if (node != NULL)
+
+    /* I'm not sure why this isn't an error.  */
+    if (!node) return;
+    rev = ((Entnode *) node->data)->version;
+
+    xchmod (filename, 1);
+    mkdir_if_needed (CVSADM_BASE);
+    basefn = make_base_file_name (filename, rev);
+
+    if (!isfile (basefn))
     {
-	finfo.file = filename;
-	finfo.fullname = short_pathname;
-	finfo.update_dir = dir_name (short_pathname);
-	base_register (&finfo, ((Entnode *) node->data)->version);
-	free ((char *)finfo.update_dir);
+	/* If this client is interoperating with clients older than 1.12.14,
+	 * then the CVS/Base/.#FILENAME.REV may not have been created on
+	 * checkout/update/commit.
+	 */
+	if (!quiet)
+	{
+	    error (0, 0, "Base revision (%s) for `%s' is missing.",
+		   rev, short_pathname);
+	    error (0, 0, "Using `%s' to create unedit fallback.",
+		   short_pathname);
+	}
+
+	free (basefn);
+	basefn = xstrdup (filename);
     }
+
+    editbasefn = Xasprintf ("%s/%s", CVSADM_BASE, filename);
+    if (isfile (editbasefn))
+	xchmod (editbasefn, true);
+    copy_file (basefn, editbasefn);
+    free (basefn);
+    free (editbasefn);
+
+    finfo.file = filename;
+    finfo.fullname = short_pathname;
+    finfo.update_dir = dir_name (short_pathname);
+    base_register (finfo.update_dir, finfo.file, rev);
+    free ((char *)finfo.update_dir);
 }
 
 
@@ -346,11 +399,9 @@ edit_fileproc (void *callerdat, struct file_info *finfo)
     char *ascnow;
     Vers_TS *vers;
 
-#if defined (CLIENT_SUPPORT) || defined (SERVER_SUPPORT)
+#if defined (CLIENT_SUPPORT)
     assert (!(current_parsed_root->isremote && check_edited));
-#else
-    assert (!check_edited);
-#endif /* defined (CLIENT_SUPPORT) || defined (SERVER_SUPPORT) */
+#endif /* CLIENT_SUPPORT */
 
     if (noexec)
 	return 0;
@@ -387,14 +438,14 @@ edit_fileproc (void *callerdat, struct file_info *finfo)
         }
     }
 
-    fp = open_file (CVSADM_NOTIFY, "a");
+    fp = xfopen (CVSADM_NOTIFY, "a");
 
     (void) time (&now);
     ascnow = asctime (gmtime (&now));
     ascnow[24] = '\0';
     /* Fix non-standard format.  */
     if (ascnow[8] == '0') ascnow[8] = ' ';
-    fprintf (fp, "E%s\t%s GMT\t%s\t%s\t", finfo->file,
+    fprintf (fp, "E%s\t%s -0000\t%s\t%s\t", finfo->file,
 	     ascnow, hostname, CurDir);
     if (setting_tedit)
 	fprintf (fp, "E");
@@ -433,14 +484,14 @@ edit_fileproc (void *callerdat, struct file_info *finfo)
 
 static const char *const edit_usage[] =
 {
-    "Usage: %s %s [-cflR] [files...]\n",
-    "-c: Check that working files are unedited\n",
-    "-f: Force edit if working files are edited (default)\n",
-    "-l: Local directory only, not recursive\n",
-    "-R: Process directories recursively\n",
-    "-a: Specify what actions for temporary watch, one of\n",
-    "    edit,unedit,commit,all,none\n",
-    "(Specify the --help global option for a list of other help options)\n",
+    "Usage: %s %s [-lRcf] [-a <action>]... [<file>]...\n",
+    "-l\tLocal directory only, not recursive.\n",
+    "-R\tProcess directories recursively (default).\n",
+    "-a\tSpecify action to register for temporary watch, one of:\n",
+    "  \t`edit', `unedit', `commit', `all', `none' (defaults to `all').\n",
+    "-c\tCheck for <file>s edited by others and abort if found.\n",
+    "-f\tAllow edit if <file>s are edited by others (default).\n",
+    "(Specify the --help global option for a list of other help options.)\n",
     NULL
 };
 
@@ -601,15 +652,12 @@ unedit_fileproc (void *callerdat, struct file_info *finfo)
     FILE *fp;
     time_t now;
     char *ascnow;
-    char *basefilename;
+    char *basefilename = NULL;
 
     if (noexec)
 	return 0;
 
-    basefilename = xmalloc (10 + sizeof CVSADM_BASE + strlen (finfo->file));
-    strcpy (basefilename, CVSADM_BASE);
-    strcat (basefilename, "/");
-    strcat (basefilename, finfo->file);
+    basefilename = Xasprintf ("%s/%s", CVSADM_BASE, finfo->file);
     if (!isfile (basefilename))
     {
 	/* This file apparently was never cvs edit'd (e.g. we are uneditting
@@ -633,14 +681,14 @@ unedit_fileproc (void *callerdat, struct file_info *finfo)
     rename_file (basefilename, finfo->file);
     free (basefilename);
 
-    fp = open_file (CVSADM_NOTIFY, "a");
+    fp = xfopen (CVSADM_NOTIFY, "a");
 
     (void) time (&now);
     ascnow = asctime (gmtime (&now));
     ascnow[24] = '\0';
     /* Fix non-standard format.  */
     if (ascnow[8] == '0') ascnow[8] = ' ';
-    fprintf (fp, "U%s\t%s GMT\t%s\t%s\t\n", finfo->file,
+    fprintf (fp, "U%s\t%s -0000\t%s\t%s\t\n", finfo->file,
 	     ascnow, hostname, CurDir);
 
     if (fclose (fp) < 0)
@@ -669,7 +717,7 @@ unedit_fileproc (void *callerdat, struct file_info *finfo)
 	Node *node;
 	Entnode *entdata;
 
-	baserev = base_get (finfo);
+	baserev = base_get (finfo->update_dir, finfo->file);
 	node = findnode_fn (finfo->entries, finfo->file);
 	/* The case where node is NULL probably should be an error or
 	   something, but I don't want to think about it too hard right
@@ -702,7 +750,7 @@ unedit_fileproc (void *callerdat, struct file_info *finfo)
 		      entdata->conflict);
 	}
 	free (baserev);
-	base_deregister (finfo);
+	base_deregister (finfo->update_dir, finfo->file);
     }
 
     xchmod (finfo->file, 0);
@@ -711,10 +759,10 @@ unedit_fileproc (void *callerdat, struct file_info *finfo)
 
 static const char *const unedit_usage[] =
 {
-    "Usage: %s %s [-lR] [files...]\n",
-    "-l: Local directory only, not recursive\n",
-    "-R: Process directories recursively\n",
-    "(Specify the --help global option for a list of other help options)\n",
+    "Usage: %s %s [-lR] [<file>]...\n",
+    "-l\tLocal directory only, not recursive.\n",
+    "-R\tProcess directories recursively (default).\n",
+    "(Specify the --help global option for a list of other help options.)\n",
     NULL
 };
 
@@ -761,18 +809,15 @@ unedit (int argc, char **argv)
 
 
 void
-mark_up_to_date (const char *file)
+mark_up_to_date (const char*update_dir, const char *file)
 {
     char *base;
 
     /* The file is up to date, so we better get rid of an out of
        date file in CVSADM_BASE.  */
-    base = xmalloc (strlen (file) + 80);
-    strcpy (base, CVSADM_BASE);
-    strcat (base, "/");
-    strcat (base, file);
+    base = Xasprintf ("%s/%s", CVSADM_BASE, file);
     if (unlink_file (base) < 0 && ! existence_error (errno))
-	error (0, errno, "cannot remove %s", file);
+	error (0, errno, "cannot remove `%s/%s'", update_dir, base);
     free (base);
 }
 
@@ -819,24 +864,28 @@ notify_proc (const char *repository, const char *filter, void *closure)
     const char *srepos = Short_Repository (repository);
     struct notify_proc_args *args = closure;
 
+    /*
+     * Cast any NULL arguments as appropriate pointers as this is an
+     * stdarg function and we need to be certain the caller gets what
+     * is expected.
+     */
     cmdline = format_cmdline (
 #ifdef SUPPORT_OLD_INFO_FMT_STRINGS
-	false, srepos,
+			      false, srepos,
 #endif /* SUPPORT_OLD_INFO_FMT_STRINGS */
-	filter,
-	"c", "s", cvs_cmd_name,
+			      filter,
+			      "c", "s", cvs_cmd_name,
 #ifdef SERVER_SUPPORT
-        "R", "s", referrer ? referrer->original : "NONE",
+			      "R", "s", referrer ? referrer->original : "NONE",
 #endif /* SERVER_SUPPORT */
-    	"p", "s", srepos,
-	"r", "s", current_parsed_root->directory,
-	"s", "s", args->notifyee,
-	NULL
-	);
-    if (!cmdline || !strlen(cmdline))
+			      "p", "s", srepos,
+			      "r", "s", current_parsed_root->directory,
+			      "s", "s", args->notifyee,
+			      (char *) NULL);
+    if (!cmdline || !strlen (cmdline))
     {
 	if (cmdline) free (cmdline);
-	error(0, 0, "pretag proc resolved to the empty string!");
+	error (0, 0, "pretag proc resolved to the empty string!");
 	return 1;
     }
 
@@ -844,7 +893,7 @@ notify_proc (const char *repository, const char *filter, void *closure)
     if (pipefp == NULL)
     {
 	error (0, errno, "cannot write entry to notify filter: %s", cmdline);
-	free(cmdline);
+	free (cmdline);
 	return 1;
     }
 
@@ -855,7 +904,7 @@ notify_proc (const char *repository, const char *filter, void *closure)
     /* Lots more potentially useful information we could add here; see
        logfile_write for inspiration.  */
 
-    free(cmdline);
+    free (cmdline);
     return pclose (pipefp);
 }
 
@@ -1012,15 +1061,8 @@ notify_do (int type, const char *filename, const char *update_dir,
 	    size_t line_len = 0;
 
 	    args.notifyee = NULL;
-	    usersname = xmalloc (strlen (current_parsed_root->directory)
-				 + sizeof CVSROOTADM
-				 + sizeof CVSROOTADM_USERS
-				 + 20);
-	    strcpy (usersname, current_parsed_root->directory);
-	    strcat (usersname, "/");
-	    strcat (usersname, CVSROOTADM);
-	    strcat (usersname, "/");
-	    strcat (usersname, CVSROOTADM_USERS);
+	    usersname = Xasprintf ("%s/%s/%s", current_parsed_root->directory,
+				   CVSROOTADM, CVSROOTADM_USERS);
 	    fp = CVS_FOPEN (usersname, "r");
 	    if (fp == NULL && !existence_error (errno))
 		error (0, errno, "cannot read %s", usersname);
@@ -1170,10 +1212,10 @@ notify_check (const char *repository, const char *update_dir)
 
 static const char *const editors_usage[] =
 {
-    "Usage: %s %s [-lR] [files...]\n",
-    "\t-l\tProcess this directory only (not recursive).\n",
-    "\t-R\tProcess directories recursively.\n",
-    "(Specify the --help global option for a list of other help options)\n",
+    "Usage: %s %s [-lR] [<file>]...\n",
+    "-l\tProcess this directory only (not recursive).\n",
+    "-R\tProcess directories recursively (default).\n",
+    "(Specify the --help global option for a list of other help options.)\n",
     NULL
 };
 
