@@ -12,6 +12,14 @@
  * Polk's hash list manager.  So cool.
  */
 
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
+
+/* Verify interface.  */
+#include "hash.h"
+
+/* CVS */
 #include "cvs.h"
 
 /* Global caches.  The idea is that we maintain a linked list of "free"d
@@ -138,6 +146,45 @@ dellist (List **listp)
 
 
 
+/*
+ * remove a node from it's list (maybe hash list too)
+ */
+void
+removenode (Node *p)
+{
+    if (!p) return;
+
+    /* take it out of the list */
+    p->next->prev = p->prev;
+    p->prev->next = p->next;
+
+    /* if it was hashed, remove it from there too */
+    if (p->hashnext)
+    {
+	p->hashnext->hashprev = p->hashprev;
+	p->hashprev->hashnext = p->hashnext;
+    }
+}
+
+
+
+void
+mergelists (List *dest, List **src)
+{
+    Node *head, *p, *n;
+
+    head = (*src)->list;
+    for (p = head->next; p != head; p = n)
+    {
+	n = p->next;
+	removenode (p);
+	addnode (dest, p);
+    }
+    dellist (src);
+}
+
+
+
 /*
  * get a new list node
  */
@@ -173,20 +220,9 @@ getnode (void)
 void
 delnode (Node *p)
 {
-    if (p == NULL)
-	return;
-
-    /* take it out of the list */
-    p->next->prev = p->prev;
-    p->prev->next = p->next;
-
-    /* if it was hashed, remove it from there too */
-    if (p->hashnext != NULL)
-    {
-	p->hashnext->hashprev = p->hashprev;
-	p->hashprev->hashnext = p->hashnext;
-    }
-
+    if (!p) return;
+    /* remove it */
+    removenode (p);
     /* free up the storage */
     freenode (p);
 }
@@ -495,6 +531,7 @@ nodetypestring (Ntype type)
     case VARIABLE:	return "VARIABLE";
     case RCSFIELD:	return "RCSFIELD";
     case RCSCMPFLD:	return "RCSCMPFLD";
+    case RCSSTRING:	return "RCSSTRING";
     }
 
     return "<trash>";
