@@ -113,9 +113,11 @@ password_entry_parseline (const char *cvsroot_canonical,
     /* look for '^/' */
     if (*linebuf == '/')
     {
-	/* Yes: slurp '^/\d+\D' and parse the rest of the line according to version number */
+	/* Yes: slurp '^/\d+\D' and parse the rest of the line according to
+	 * version number
+	 */
 	char *q;
-	unsigned long int entry_version;
+	unsigned long int entry_version = 0 /* Placate -Wall.  */;
 
 	if (isspace(*(linebuf + 1)))
 	    /* special case since strtoul ignores leading white space */
@@ -123,12 +125,10 @@ password_entry_parseline (const char *cvsroot_canonical,
 	else
 	    entry_version = strtoul (linebuf + 1, &q, 10);
 
-	if (q == linebuf + 1)
-	    /* no valid digits found by strtoul */
-	    entry_version = 0;
-	else
+	if (q != linebuf + 1)
 	    /* assume a delimiting seperator */
 	    q++;
+	/* else, no valid digits found by strtoul */
 
 	switch (entry_version)
 	{
@@ -383,7 +383,8 @@ process:
 
 	/* create and open a temp file */
 	if ((tmp_fp = cvs_temp_file (&tmp_name)) == NULL)
-	    error (1, errno, "unable to open temp file %s", tmp_name);
+	    error (1, errno, "unable to open temp file %s",
+		   tmp_name ? tmp_name : "(null)");
 
 	line = 0L;
 	while ((line_length = getline (&linebuf, &linebuf_len, fp)) >= 0)
@@ -571,9 +572,10 @@ login (int argc, char **argv)
 
 
 
-/* Returns the _scrambled_ password.  The server must descramble
-   before hashing and comparing.  If password file not found, or
-   password not found in the file, just return NULL. */
+/* Returns the _scrambled_ password in freshly allocated memory.  The server
+ * must descramble before hashing and comparing.  If password file not found,
+ * or password not found in the file, just return NULL.
+ */
 char *
 get_cvs_password (void)
 {
@@ -584,7 +586,7 @@ get_cvs_password (void)
        context, then assume they have supplied the correct, scrambled
        password. */
     if (cvs_password)
-	return cvs_password;
+	return xstrdup (cvs_password);
 
     if (getenv ("CVS_PASSWORD") != NULL)
     {
