@@ -1,10 +1,14 @@
 /*
- *    Copyright (c) 1992, Brian Berliner and Jeff Polk
- *    Copyright (c) 1989-1992, Brian Berliner
+ * Copyright (C) 1986-2006 The Free Software Foundation, Inc.
  *
- *    You may distribute under the terms of the GNU General Public License
- *    as specified in the README file that comes with the CVS source
- *    distribution.
+ * Portions Copyright (C) 1998-2006 Derek Price, Ximbiot <http://ximbiot.com>,
+ *                                  and others.
+ *
+ * Portions Copyright (C) 1992, Brian Berliner and Jeff Polk
+ * Portions Copyright (C) 1989-1992, Brian Berliner
+ *
+ * You may distribute under the terms of the GNU General Public License
+ * as specified in the README file that comes with the CVS source distribution.
  *
  * This is the main C driver for the CVS system.
  *
@@ -13,38 +17,51 @@
  *
  */
 
-#include "cvs.h"
-#include "strftime.h"
-
-#ifdef HAVE_WINSOCK_H
-# include <winsock.h>
-#elif !HAVE_GETHOSTNAME
-extern int gethostname (char *, int);
+#ifdef HAVE_CONFIG_H
+# include <config.h>
 #endif
 
+/* GNULIB Headers.  */
+#include "closeout.h"
+#include "setenv.h"
+#include "strftime.h"
+#include "xgethostname.h"
+
+/* CVS Headers.  */
+#include "command_line_opt.h"
+#include "gpg.h"
+#include "sign.h"
+#include "verify.h"
+
+#include "cvs.h"
+
+
+
 const char *program_name;
 const char *program_path;
 const char *cvs_cmd_name;
 
-/* I'd dynamically allocate this, but it seems like gethostname
-   requires a fixed size array.  If I'm remembering the RFCs right,
-   256 should be enough.  */
-#ifndef MAXHOSTNAMELEN
-#define MAXHOSTNAMELEN  256
-#endif
+const char *global_session_id; /* Random session ID */
 
-char hostname[MAXHOSTNAMELEN];
+char *hostname;
+/* FIXME: Perhaps this should be renamed original_hostname or the like?  */
+char *server_hostname;
 
 int use_editor = 1;
 int use_cvsrc = 1;
 int cvswrite = !CVSREAD_DFLT;
 int really_quiet = 0;
 int quiet = 0;
-int trace = 0;
 int noexec = 0;
 int readonlyfs = 0;
 int logoff = 0;
+long connection_timeout = 0;
 
+/*
+ * Zero if compression isn't supported or requested; non-zero to indicate
+ * a compression level to request from gzip.
+ */
+int gzip_level;
 
 
 /***
@@ -55,18 +72,51 @@ int logoff = 0;
 struct config *config;
 
 
-
+bool suppress_bases = false;
 mode_t cvsumask = UMASK_DFLT;
-
 char *CurDir;
 
 /*
  * Defaults, for the environment variables that are not set
  */
-char *Tmpdir = TMPDIR_DFLT;
 char *Editor = EDITOR_DFLT;
 
 
+
+/* Temp dir stuff.  */
+
+/* Temp dir, if set by the user.  */
+static char *tmpdir_cmdline;
+
+
+
+/* Returns in order of precedence:
+ *
+ *	1.  Temp dir as set via the command line.
+ *	2.  Temp dir as set in CVSROOT/config.
+ *	3.  Temp dir as set in $TMPDIR env var.
+ *	4.  Contents of TMPDIR_DFLT preprocessor macro.
+ *
+ * ERRORS
+ *  It is a fatal error if this function would otherwise return NULL or an
+ *  empty string.
+ */
+const char *
+get_cvs_tmp_dir (void)
+{
+    const char *retval;
+    if (tmpdir_cmdline) retval = tmpdir_cmdline;
+    else if (config && config->TmpDir) retval = config->TmpDir;
+    else retval = get_system_temp_dir ();
+    if (!retval) retval = TMPDIR_DFLT;
+
+    if (!retval || !*retval) error (1, 0, "No temp dir specified.");
+
+    return retval;
+}
+
+
+
 /* When our working directory contains subdirectories with different
    values in CVS/Root files, we maintain a list of them.  */
 List *root_directories = NULL;
@@ -137,10 +187,12 @@ static const struct cmd
 #ifdef SERVER_SUPPORT
     { "server",   NULL,       NULL,        server,    CVS_CMD_MODIFIES_REPOSITORY | CVS_CMD_USES_WORK_DIR },
 #endif
+    { "sign",   "sig",        NULL,        sign,      0 },
     { "status",   "st",       "stat",      cvsstatus, CVS_CMD_USES_WORK_DIR },
     { "tag",      "ta",       "freeze",    cvstag,    CVS_CMD_MODIFIES_REPOSITORY | CVS_CMD_USES_WORK_DIR },
     { "unedit",   NULL,       NULL,        unedit,    CVS_CMD_MODIFIES_REPOSITORY | CVS_CMD_USES_WORK_DIR },
     { "update",   "up",       "upd",       update,    CVS_CMD_USES_WORK_DIR },
+    { "verify",   "ver",      NULL,        verify,    0 },
     { "version",  "ve",       "ver",       version,   0 },
     { "watch",    NULL,       NULL,        watch,     CVS_CMD_MODIFIES_REPOSITORY | CVS_CMD_USES_WORK_DIR },
     { "watchers", NULL,       NULL,        watchers,  CVS_CMD_USES_WORK_DIR },
@@ -188,8 +240,8 @@ static const char *const usg[] =
        version control means.  */
 
     "For CVS updates and additional information, see\n",
-    "    the CVS home page at http://www.cvshome.org/ or\n",
-    "    Pascal Molli's CVS site at http://www.loria.fr/~molli/cvs-index.html\n",
+    "    the CVS home page at http://www.nongnu.org/cvs/ or\n",
+    "    the CVSNT home page at http://www.cvsnt.org/\n",
     NULL,
 };
 
@@ -230,10 +282,12 @@ static const char *const cmd_usage[] =
 #ifdef SERVER_SUPPORT
     "        server       Server mode\n",
 #endif
+    "        sign         Add an OpenPGP signature to one or more revisions\n",
     "        status       Display status information on checked out files\n",
     "        tag          Add a symbolic tag to checked out version of files\n",
     "        unedit       Undo an edit command\n",
     "        update       Bring work tree in sync with repository\n",
+    "        verify       Verify OpenPGP signature(s)\n",
     "        version      Show current CVS version(s)\n",
     "        watch        Set watches\n",
     "        watchers     See who is watching a file\n",
@@ -253,6 +307,8 @@ static const char *const opt_usage[] =
     "    -n           Do not execute anything that will change the disk.\n",
     "    -t           Show trace of program execution (repeat for more\n",
     "                 verbosity) -- try with -n.\n",
+    "    --timeout WAITFOR\n",
+    "                 Time out network connections in WAITFOR seconds.\n",
     "    -R           Assume repository is read-only, such as CDROM\n",
     "    -v           CVS version and copyright.\n",
     "    -T tmpdir    Use 'tmpdir' for temporary files.\n",
@@ -260,13 +316,35 @@ static const char *const opt_usage[] =
     "    -d CVS_root  Overrides $CVSROOT as the root of the CVS tree.\n",
     "    -f           Do not use the ~/.cvsrc file.\n",
 #ifdef CLIENT_SUPPORT
-    "    -z #         Use compression level '#' for net traffic.\n",
+    "    -B           Suppress use of base files.\n",
+    "    -z #         Request compression level '#' for net traffic.\n",
 #ifdef ENCRYPTION
     "    -x           Encrypt all net traffic.\n",
 #endif
     "    -a           Authenticate all net traffic.\n",
 #endif
     "    -s VAR=VAL   Set CVS user variable.\n",
+    "\n",
+    "    -g           Force OpenPGP commit signatures (default autonegotiates).\n",
+    "    --sign[=(on | off | auto)] | --no-sign\n",
+    "                 Force (or forbid) OpenPGP commit signatures\n",
+    "                 (default autonegotiates).\n",
+    "    -G TEMPLATE\n",
+    "    --sign-template TEMPLATE\n",
+    "                 Use TEMPLATE to generate OpenPGP signatures.\n",
+    "    --sign-arg ARG\n",
+    "                 Pass ARG to OpenPGP TEMPLATE when sigining.\n",
+    "    --openpgp-textmode ARG\n",
+    "                 Pass ARG to OpenPGP TEMPLATE when verifying or\n",
+    "                 generating signatures.\n",
+    "    --verify[=(off | warn | fatal)] | --no-verify\n",
+    "                 Force (or forbid) OpenPGP signature verification\n",
+    "                 on checkout (default warns on failure).\n",
+    "    -G TEMPLATE\n",
+    "    --verify-template TEMPLATE\n",
+    "                 Use TEMPLATE to verify OpenPGP signatures.\n",
+    "    --verify-arg ARG\n",
+    "                 Pass ARG to OpenPGP TEMPLATE when verifying.\n",
     "(Specify the --help option for a list of other help options)\n",
     NULL
 };
@@ -300,20 +378,16 @@ cmd_synonyms (void)
 	c++;
     }
     
-    synonyms = (char **) xmalloc(numcmds * sizeof(char *));
+    synonyms = xnmalloc (numcmds, sizeof(char *));
     line = synonyms;
     *line++ = "CVS command synonyms are:\n";
     for (c = &cmds[0]; c->fullname != NULL; c++)
     {
 	if (c->nick1 || c->nick2)
 	{
-	    *line = xmalloc (strlen (c->fullname)
-			     + (c->nick1 != NULL ? strlen (c->nick1) : 0)
-			     + (c->nick2 != NULL ? strlen (c->nick2) : 0)
-			     + 40);
-	    sprintf(*line, "        %-12s %s %s\n", c->fullname,
-		    c->nick1 ? c->nick1 : "",
-		    c->nick2 ? c->nick2 : "");
+	    *line = Xasprintf ("        %-12s %s %s\n", c->fullname,
+			       c->nick1 ? c->nick1 : "",
+			       c->nick2 ? c->nick2 : "");
 	    line++;
 	}
     }
@@ -420,6 +494,55 @@ main_cleanup (int sig)
 
 
 
+/* From server.c.
+ *
+ * When !defined ALLOW_CONFIG_OVERRIDE, this will never have any value but
+ * NULL.
+ */
+extern char *gConfigPath;
+
+
+
+
+enum {RANDOM_BYTES = 8};
+enum {COMMITID_RAW_SIZE = (sizeof(time_t) + RANDOM_BYTES)};
+
+static char const alphabet[62] =
+  "0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";
+
+/* Divide BUF by D, returning the remainder.  Replace BUF by the
+   quotient.  BUF[0] is the most significant part of BUF.
+   D must not exceed UINT_MAX >> CHAR_BIT.  */
+static unsigned int
+divide_by (unsigned char buf[COMMITID_RAW_SIZE], unsigned int d)
+{
+    unsigned int carry = 0;
+    int i;
+    for (i = 0; i < COMMITID_RAW_SIZE; i++)
+    {
+	unsigned int byte = buf[i];
+	unsigned int dividend = (carry << CHAR_BIT) + byte;
+	buf[i] = dividend / d;
+	carry = dividend % d;
+    }
+    return carry;
+}
+
+static void
+convert (char const input[COMMITID_RAW_SIZE], char *output)
+{
+    static char const zero[COMMITID_RAW_SIZE] = { 0, };
+    unsigned char buf[COMMITID_RAW_SIZE];
+    size_t o = 0;
+    memcpy (buf, input, COMMITID_RAW_SIZE);
+    while (memcmp (buf, zero, COMMITID_RAW_SIZE) != 0)
+	output[o++] = alphabet[divide_by (buf, sizeof alphabet)];
+    if (! o)
+	output[o++] = '0';
+    output[o] = '\0';
+}
+
+
 int
 main (int argc, char **argv)
 {
@@ -428,14 +551,12 @@ main (int argc, char **argv)
     char *cp, *end;
     const struct cmd *cm;
     int c, err = 0;
-    int tmpdir_update_env;
     int free_Editor = 0;
-    int free_Tmpdir = 0;
 
     int help = 0;		/* Has the user asked for help?  This
 				   lets us support the `cvs -H cmd'
 				   convention to give help for cmd. */
-    static const char short_options[] = "+QqrwtnRvb:T:e:d:Hfz:s:xa";
+    static const char short_options[] = "+QBqrwtnRvb:T:e:d:Hfz:s:xag::G:";
     static struct option long_options[] =
     {
         {"help", 0, NULL, 'H'},
@@ -443,8 +564,19 @@ main (int argc, char **argv)
 	{"help-commands", 0, NULL, 1},
 	{"help-synonyms", 0, NULL, 2},
 	{"help-options", 0, NULL, 4},
+	{"sign", optional_argument, NULL, 'g'},
+	{"no-sign", 0, NULL, 5},
+	{"sign-template", required_argument, NULL, 'G'},
+	{"sign-arg", required_argument, NULL, 6},
+	{"openpgp-textmode", required_argument, NULL, 7},
+	{"no-openpgp-textmode", 0, NULL, 8},
+	{"verify", optional_argument, NULL, 9},
+	{"no-verify", 0, NULL, 10},
+	{"verify-template", required_argument, NULL, 11},
+	{"verify-arg", required_argument, NULL, 12},
 #ifdef SERVER_SUPPORT
 	{"allow-root", required_argument, NULL, 3},
+	{"timeout", required_argument, NULL, 13},
 #endif /* SERVER_SUPPORT */
         {0, 0, 0, 0}
     };
@@ -471,8 +603,9 @@ main (int argc, char **argv)
 #endif
 
     /*
-     * Just save the last component of the path for error messages
+     * Initialize globals.
      */
+    /* Just save the last component of the path for error messages.  */
     program_path = xstrdup (argv[0]);
 #ifdef ARGV0_NOT_PROGRAM_NAME
     /* On some systems, e.g. VMS, argv[0] is not the name of the command
@@ -482,16 +615,11 @@ main (int argc, char **argv)
     program_name = last_component (argv[0]);
 #endif
 
+
     /*
      * Query the environment variables up-front, so that
      * they can be overridden by command line arguments
      */
-    tmpdir_update_env = *Tmpdir;	/* TMPDIR_DFLT must be set */
-    if ((cp = getenv (TMPDIR_ENV)) != NULL)
-    {
-	Tmpdir = cp;
-	tmpdir_update_env = 0;		/* it's already there */
-    }
     if ((cp = getenv (EDITOR1_ENV)) != NULL)
  	Editor = cp;
     else if ((cp = getenv (EDITOR2_ENV)) != NULL)
@@ -504,6 +632,53 @@ main (int argc, char **argv)
 	readonlyfs = 1;
 	logoff = 1;
     }
+    if ((cp = getenv (CVS_VERIFY_CHECKOUTS_ENV)))
+    {
+	if (!strcasecmp (cp, "warn"))
+	    set_verify_checkouts (VERIFY_WARN);
+	else if (!strcasecmp (cp, "fatal"))
+	    set_verify_checkouts (VERIFY_FATAL);
+	else
+	{
+	    bool on;
+	    if (readBool ("environment", CVS_VERIFY_CHECKOUTS_ENV, cp, &on))
+	    {
+		if (on)
+		    set_verify_checkouts (VERIFY_FATAL);
+		else
+		    set_verify_checkouts (VERIFY_OFF);
+	    }
+	    else
+		error (1, 0,
+		       "Unrecognized content (`%s') in $%s",
+		       cp, CVS_VERIFY_CHECKOUTS_ENV);
+	}
+    }
+    if ((cp = getenv (CVS_SIGN_COMMITS_ENV)))
+    {
+	if (!strcasecmp (cp, "auto")
+	    || !strcasecmp (cp, "server"))
+	    set_sign_commits (SIGN_DEFAULT);
+	else if (!strcasecmp (cp, ""))
+	    set_sign_commits (SIGN_NEVER);
+	else
+	{
+	    bool on;
+	    if (readBool ("environment", CVS_SIGN_COMMITS_ENV, cp, &on))
+	    {
+		if (on)
+		    set_sign_commits (SIGN_ALWAYS);
+		else
+		    set_sign_commits (SIGN_NEVER);
+	    }
+	    else
+		error (0, 0,
+		       "Unrecognized content (`%s') in $%s ignored",
+		       cp, CVS_SIGN_COMMITS_ENV);
+	}
+    } 
+    if ((cp = getenv (CVS_VERIFY_TEMPLATE_ENV)))
+	set_verify_template (cp);
 
     /* Set this to 0 to force getopt initialization.  getopt() sets
        this to 1 internally.  */
@@ -523,6 +698,26 @@ main (int argc, char **argv)
 	    use_cvsrc = 0;
     }
 
+#ifdef SERVER_SUPPORT
+    /* Don't try and read a .cvsrc file if we are a server.  */
+    if (optind < argc
+	&& (false
+# if defined (AUTH_SERVER_SUPPORT) || defined (HAVE_GSSAPI)
+	    || !strcmp (argv[optind], "pserver")
+# endif
+# ifdef HAVE_KERBEROS
+	    || !strcmp (argv[optind], "kserver")
+# endif /* HAVE_KERBEROS */
+	    || !strcmp (argv[optind], "server")))
+	{
+	    /* Avoid any .cvsrc file.  */
+	    use_cvsrc = 0;
+	    /* Pre-parse the server options to get the config path.  */
+	    cvs_cmd_name = argv[optind];
+	    parseServerOptions (argc - optind, argv + optind);
+	}
+#endif /* SERVER_SUPPORT */
+
     /*
      * Scan cvsrc file for global options.
      */
@@ -550,10 +745,114 @@ main (int argc, char **argv)
 		/* --help-options */
 		usage (opt_usage);
 		break;
+	    case 'g':
+		/* --sign */
+		if (optarg)
+		{
+		    if (!strcasecmp (optarg, "auto")
+			|| !strcasecmp (optarg, "server"))
+			set_sign_commits (SIGN_DEFAULT);
+		    else if (!strcasecmp (optarg, "on"))
+			set_sign_commits (SIGN_ALWAYS);
+		    else if (!strcasecmp (optarg, "off"))
+			set_sign_commits (SIGN_NEVER);
+		    else
+			error (1, 0, "Unrecognized argument to --sign (`%s')",
+			       optarg);
+		}
+		else
+		    set_sign_commits (SIGN_ALWAYS);
+		break;
+	    case 5:
+		/* --no-sign */
+		set_sign_commits (SIGN_NEVER);
+		break;
+	    case 'G':
+		/* --sign-template */
+		set_sign_template (optarg);
+		break;
+	    case 6:
+		/* --sign-arg */
+		add_sign_arg (optarg);
+		break;
+	    case 7:
+		/* --openpgp-textmode */
+		set_openpgp_textmode (optarg);
+		break;
+	    case 8:
+		/* --no-openpgp-textmode */
+		set_openpgp_textmode ("");
+		break;
+	    case 9:
+		/* --verify */
+		if (optarg)
+		{
+		    if (!strcasecmp (optarg, "off")
+			|| !strcasecmp (optarg, "never")
+			|| !strcasecmp (optarg, "false"))
+			set_verify_checkouts (VERIFY_OFF);
+		    else if (!strcasecmp (optarg, "warn"))
+			set_verify_checkouts (VERIFY_WARN);
+		    else if (!strcasecmp (optarg, "always")
+			     || !strcasecmp (optarg, "fatal")
+			     || !strcasecmp (optarg, "on")
+			     || !strcasecmp (optarg, "true"))
+			set_verify_checkouts (VERIFY_FATAL);
+		    else
+			error (1, 0,
+			       "Unrecognized argument to --verify (`%s')",
+			       optarg);
+		}
+		else
+		    set_verify_checkouts (VERIFY_FATAL);
+		break;
+	    case 10:
+	        /* --no-verify */
+		set_verify_checkouts (VERIFY_OFF);
+		break;
+	    case 11:
+		/* --verify-template */
+		set_verify_template (optarg);
+		break;
+	    case 12:
+		/* --verify-arg */
+		add_verify_arg (optarg);
+		break;
 #ifdef SERVER_SUPPORT
 	    case 3:
 		/* --allow-root */
-		root_allow_add (optarg);
+		root_allow_add (optarg, gConfigPath);
+		break;
+	    case 13:
+		/* --timeout */
+		connection_timeout = strtol (optarg, &end, 10);
+		if (*end != '\0')
+		{
+		    char tmp = *end++;
+		    if (*end != '\0')
+			tmp = '?';
+		    switch (tmp)
+		    {
+			case 'd':
+			    connection_timeout *= 24;
+			case 'h':
+			    connection_timeout *= 60;
+			case 'm':
+			    connection_timeout *= 60;
+			case 's':
+			    break;
+			default:
+			    error (0, 0,
+"unknown units (`%s') in argument to --timeout",
+				   end - 1);
+			    error (1, 0,
+"(valid units are: none, `d', `h', `m', & `s')");
+			    break;
+		    }
+		}
+		if (connection_timeout < 0)
+		  error (1, 0,
+"argument to --timeout must be greater than or equal to 0");
 		break;
 #endif /* SERVER_SUPPORT */
 	    case 'Q':
@@ -580,17 +879,22 @@ main (int argc, char **argv)
 		logoff = 1;
 		break;
 	    case 'v':
-		fputs ("\n", stdout);
+		(void) fputs ("\n", stdout);
 		version (0, NULL);    
-		fputs (
-"\n"
-"Copyright (c) 1989-2004 Brian Berliner, david d `zoo' zuhn,\n"
-"                        Jeff Polk, and other authors\n"
-"\n"
-"CVS may be copied only under the terms of the GNU General Public License,\n"
-"a copy of which can be found with the CVS distribution kit.\n"
-"\n"
-"Specify the --help option for further information about CVS\n", stdout);
+		(void) fputs ("\n", stdout);
+		(void) fputs ("\
+Copyright (C) 2006 Free Software Foundation, Inc.\n\
+\n\
+Senior active maintainers include Larry Jones, Derek R. Price,\n\
+and Mark D. Baushke.  Please see the AUTHORS and README files from the CVS\n\
+distribution kit for a complete list of contributors and copyrights.\n",
+		              stdout);
+		(void) fputs ("\n", stdout);
+		(void) fputs ("CVS may be copied only under the terms of the GNU General Public License,\n", stdout);
+		(void) fputs ("a copy of which can be found with the CVS distribution kit.\n", stdout);
+		(void) fputs ("\n", stdout);
+
+		(void) fputs ("Specify the --help option for further information about CVS\n", stdout);
 
 		exit (0);
 		break;
@@ -602,10 +906,8 @@ main (int argc, char **argv)
 		   either new or old CVS.  */
 		break;
 	    case 'T':
-		if (free_Tmpdir) free (Tmpdir);
-		Tmpdir = xstrdup (optarg);
-		free_Tmpdir = 1;
-		tmpdir_update_env = 1;	/* need to update environment */
+		if (tmpdir_cmdline) free (tmpdir_cmdline);
+		tmpdir_cmdline = xstrdup (optarg);
 		break;
 	    case 'e':
 		if (free_Editor) free (Editor);
@@ -624,12 +926,10 @@ main (int argc, char **argv)
 		use_cvsrc = 0; /* unnecessary, since we've done it above */
 		break;
 	    case 'z':
-#ifdef CLIENT_SUPPORT
 		gzip_level = strtol (optarg, &end, 10);
 		if (*end != '\0' || gzip_level < 0 || gzip_level > 9)
 		  error (1, 0,
 			 "gzip compression level must be between 0 and 9");
-#endif /* CLIENT_SUPPORT */
 		/* If no CLIENT_SUPPORT, we just silently ignore the gzip
 		 * level, so that users can have it in their .cvsrc and not
 		 * cause any trouble.
@@ -639,6 +939,9 @@ main (int argc, char **argv)
 		 * `cvs -z -n up' which read -n as the argument to -z without
 		 * complaining.  */
 		break;
+	    case 'B':
+		suppress_bases = true;
+		break;
 	    case 's':
 		variable_set (optarg);
 		break;
@@ -678,6 +981,52 @@ Using this option to access a repository which some users write to may\n\
 cause intermittent sandbox corruption.");
     }
 
+    /* Calculate the cvs global session ID */
+
+    {
+	char buf[COMMITID_RAW_SIZE] = { 0, };
+	char out[COMMITID_RAW_SIZE * 2];
+	ssize_t len = 0;
+	time_t rightnow = time (NULL);
+	char *startrand = buf + sizeof (time_t);
+	unsigned char *p = (unsigned char *) startrand;
+	size_t randbytes = RANDOM_BYTES;
+	int flags = O_RDONLY;
+	int fd;
+#ifdef O_NOCTTY
+	flags |= O_NOCTTY;
+#endif
+	if (rightnow != (time_t)-1)
+		while (rightnow > 0) {
+		    *--p = rightnow % (UCHAR_MAX + 1);
+		    rightnow /= UCHAR_MAX + 1;
+		}
+	else {
+	    /* try to use more random data */
+	    randbytes = COMMITID_RAW_SIZE;
+	    startrand = buf;
+	}
+	fd = open ("/dev/urandom", flags);
+	if (fd >= 0) {
+	    len = read (fd, startrand, randbytes);
+	    close (fd);
+	}
+	if (len <= 0) {
+	    /* no random data was available so use pid */
+	    long int pid = (long int)getpid ();
+	    p = (unsigned char *) (startrand + sizeof (pid));
+	    while (pid > 0) {
+		*--p = pid % (UCHAR_MAX + 1);
+		pid /= UCHAR_MAX + 1;
+	    }
+	}
+	convert(buf, out);
+	global_session_id = strdup (out);
+    }
+
+
+    TRACE (TRACE_FUNCTION, "main: Session ID is %s", global_session_id);
+
     /* Look up the command name. */
 
     cvs_cmd_name = argv[0];
@@ -726,6 +1075,26 @@ cause intermittent sandbox corruption.");
 		       CVSUMASK_ENV, cp);
 	}
 
+	if (getenv (CVSNOBASES_ENV))
+	    suppress_bases = true;
+
+	/* HOSTNAME & SERVER_HOSTNAME need to be set before they are
+	 * potentially used in gserver_authenticate_connection() (called from
+	 * pserver_authenticate_connection, below).
+	 */
+	hostname = xgethostname ();
+	if (!hostname)
+	{
+            error (0, errno,
+                   "xgethostname () returned NULL, using \"localhost\"");
+            hostname = xstrdup ("localhost");
+	}
+
+	/* Keep track of this separately since the client can change
+	 * HOSTNAME on the server.
+	 */
+	server_hostname = xstrdup (hostname);
+
 #ifdef SERVER_SUPPORT
 
 # ifdef HAVE_KERBEROS
@@ -742,15 +1111,11 @@ cause intermittent sandbox corruption.");
 	}
 # endif /* HAVE_KERBEROS */
 
-
 # if defined (AUTH_SERVER_SUPPORT) || defined (HAVE_GSSAPI)
 	if (strcmp (cvs_cmd_name, "pserver") == 0)
 	{
 	    /* The reason that --allow-root is not a command option
-	       is mainly the comment in server() about how argc,argv
-	       might be from .cvsrc.  I'm not sure about that, and
-	       I'm not sure it is only true of command options, but
-	       it seems easier to make it a global option.  */
+	       is mainly that it seems easier to make it a global option.  */
 
 	    /* Gets username and password from client, authenticates, then
 	       switches to run as that user and sends an ACK back to the
@@ -761,11 +1126,9 @@ cause intermittent sandbox corruption.");
 	    cvs_cmd_name = "server";
 	}
 # endif /* AUTH_SERVER_SUPPORT || HAVE_GSSAPI */
-
-	server_active = strcmp (cvs_cmd_name, "server") == 0;
-
 #endif /* SERVER_SUPPORT */
 
+	server_active = strcmp (cvs_cmd_name, "server") == 0;
 
 #ifdef SERVER_SUPPORT
 	if (server_active)
@@ -780,56 +1143,39 @@ cause intermittent sandbox corruption.");
 	else
 #endif
 	{
+	    cleanup_register (close_stdout);
 	    CurDir = xgetcwd ();
             if (CurDir == NULL)
 		error (1, errno, "cannot get working directory");
 	}
 
-	if (Tmpdir == NULL || Tmpdir[0] == '\0')
-	    Tmpdir = "/tmp";
-
-#ifdef HAVE_PUTENV
-	if (tmpdir_update_env)
 	{
-	    char *env;
-	    env = xmalloc (strlen (TMPDIR_ENV) + strlen (Tmpdir) + 1 + 1);
-	    (void) sprintf (env, "%s=%s", TMPDIR_ENV, Tmpdir);
-	    (void) putenv (env);
-	    /* do not free env, as putenv has control of it */
-	}
-	{
-	    char *env;
+	    char *val;
 	    /* XXX pid < 10^32 */
-	    env = xmalloc (strlen (CVS_PID_ENV) + 1 + 32 + 1);
-	    (void) sprintf (env, "%s=%ld", CVS_PID_ENV, (long) getpid ());
-	    (void) putenv (env);
-	    /* do not free env, as putenv has control of it */
+	    val = Xasprintf ("%ld", (long) getpid ());
+	    setenv (CVS_PID_ENV, val, 1);
+	    free (val);
 	}
-#endif
 
 	/* make sure we clean up on error */
 	signals_register (main_cleanup);
 
-	gethostname(hostname, sizeof (hostname));
-
 #ifdef KLUDGE_FOR_WNT_TESTSUITE
 	/* Probably the need for this will go away at some point once
 	   we call fflush enough places (e.g. fflush (stdout) in
 	   cvs_outerr).  */
-	(void) setvbuf (stdout, (char *) NULL, _IONBF, 0);
-	(void) setvbuf (stderr, (char *) NULL, _IONBF, 0);
+	(void) setvbuf (stdout, NULL, _IONBF, 0);
+	(void) setvbuf (stderr, NULL, _IONBF, 0);
 #endif /* KLUDGE_FOR_WNT_TESTSUITE */
 
 	if (use_cvsrc)
 	    read_cvsrc (&argc, &argv, cvs_cmd_name);
 
-#ifdef SERVER_SUPPORT
 	/* Fiddling with CVSROOT doesn't make sense if we're running
 	 * in server mode, since the client will send the repository
 	 * directory after the connection is made.
 	 */
 	if (!server_active)
-#endif
 	{
 	    /* First check if a root was set via the command line.  */
 	    if (CVSroot_cmdline)
@@ -916,20 +1262,14 @@ cause intermittent sandbox corruption.");
 	   once).  To get out of the loop, we perform a "break" at the
 	   end of things.  */
 
-	while (
-#ifdef SERVER_SUPPORT
-	       server_active ||
-#endif
-	       walklist (root_directories, set_root_directory, NULL)
-	       )
+	while (server_active ||
+	       walklist (root_directories, set_root_directory, NULL))
 	{
-#ifdef SERVER_SUPPORT
 	    /* Fiddling with CVSROOT doesn't make sense if we're running
 	       in server mode, since the client will send the repository
 	       directory after the connection is made. */
 
 	    if (!server_active)
-#endif
 	    {
 		/* Now we're 100% sure that we have a valid CVSROOT
 		   variable.  Parse it to see if we're supposed to do
@@ -943,9 +1283,7 @@ cause intermittent sandbox corruption.");
 		/*
 		 * Check to see if the repository exists.
 		 */
-#ifdef CLIENT_SUPPORT
 		if (!current_parsed_root->isremote)
-#endif	/* CLIENT_SUPPORT */
 		{
 		    char *path;
 		    int save_errno;
@@ -963,23 +1301,9 @@ cause intermittent sandbox corruption.");
 		    free (path);
 		}
 
-#ifdef HAVE_PUTENV
 		/* Update the CVSROOT environment variable.  */
 		if (cvsroot_update_env)
-		{
-		    static char *prev;
-		    char *env;
-
-		    env = Xasprintf ("%s=%s", CVSROOT_ENV,
-				     current_parsed_root->original);
-		    (void) putenv (env);
-		    /* do not free env yet, as putenv has control of it */
-		    /* but do free the previous value, if any */
-		    if (prev != NULL)
-			free (prev);
-		    prev = env;
-		}
-#endif
+		    setenv (CVSROOT_ENV, current_parsed_root->original, 1);
 	    }
 	
 	    /* Parse the CVSROOT/config file, but only for local.  For the
@@ -989,21 +1313,19 @@ cause intermittent sandbox corruption.");
 	       predetermine whether CVSROOT/config overrides things from
 	       read_cvsrc and other such places or vice versa.  That sort
 	       of thing probably needs more thought.  */
-	    if (1
-#ifdef SERVER_SUPPORT
-		&& !server_active
-#endif
-#ifdef CLIENT_SUPPORT
-		&& !current_parsed_root->isremote
-#endif
-		)
+	    if (!server_active && !current_parsed_root->isremote)
 	    {
 		/* If there was an error parsing the config file, parse_config
 		   already printed an error.  We keep going.  Why?  Because
 		   if we didn't, then there would be no way to check in a new
 		   CVSROOT/config file to fix the broken one!  */
 		if (config) free_config (config);
-		config = parse_config (current_parsed_root->directory);
+		config = parse_config (current_parsed_root->directory, NULL);
+
+		/* Can set TMPDIR in the environment if necessary now, since
+		 * if it was set in config, we now know it.
+		 */
+		push_env_temp_dir ();
 	    }
 
 #ifdef CLIENT_SUPPORT
@@ -1047,23 +1369,18 @@ cause intermittent sandbox corruption.");
                active, our list will be empty -- don't try and
                remove it from the list. */
 
-#ifdef SERVER_SUPPORT
 	    if (!server_active)
-#endif /* SERVER_SUPPORT */
 	    {
 		Node *n = findnode (root_directories,
 				    original_parsed_root->original);
 		assert (n != NULL);
 		assert (n->data != NULL);
-		free_cvsroot_t (n->data);
 		n->data = NULL;
 		current_parsed_root = NULL;
 	    }
 
-#ifdef SERVER_SUPPORT
 	    if (server_active)
 		break;
-#endif
 	} /* end of loop for cvsroot values */
 
 	dellist (&root_directories);
@@ -1081,7 +1398,7 @@ cause intermittent sandbox corruption.");
 
 
 char *
-Make_Date (char *rawdate)
+Make_Date (const char *rawdate)
 {
     struct timespec t;
 
@@ -1094,6 +1411,68 @@ Make_Date (char *rawdate)
 
 
 
+/* Parse a string of the form TAG[:DATE], where TAG could be the empty string.
+ *
+ * INPUTS
+ *   input	The string to be parsed.
+ *
+ * OUTPUTS
+ *   tag	The tag found, if any.  If TAG is the empty string, then leave
+ *		this value unchanged.
+ *   date	The date found, if any.  If DATE is the empty string or is
+ *		missing, leave this value unchanged.
+ *
+ * NOTES
+ *   If either TAG or DATE is replaced for output, the previous value is freed.
+ *
+ * ERRORS
+ *   If either TAG or DATE cannot be parsed, then this function will exit with
+ *   a fatal error message.
+ *
+ * RETURNS
+ *   Nothing.
+ */
+void
+parse_tagdate (char **tag, char **date, const char *input)
+{
+    char *p;
+
+    TRACE (TRACE_FUNCTION, "parse_tagdate (%s, %s, %s)",
+	   *tag ? *tag : "(null)", *date ? *date : "(null)",
+	   input);
+
+    if ((p = strchr (input, ':')))
+    {
+	/* Parse the tag.  */
+	if (p - input)
+	{
+	    /* The tag has > 0 length.  */
+	    if (*tag) free (*tag);
+	    *tag = xmalloc (p - input + 1);
+	    strncpy (*tag, input, p - input);
+	    (*tag)[p - input] = '\0';
+	}
+
+	/* Parse the date.  */
+	if (*++p)
+	{
+	    if (*date) free (*date);
+	    *date = Make_Date (p);
+	}
+    }
+    else if (strlen (input))
+    {
+	/* The tag has > 0 length.  */
+	if (*tag) free (*tag);
+	*tag = xstrdup (input);
+    }
+
+    TRACE (TRACE_DATA, "parse_tagdate: got tag = `%s', date = `%s'",
+	   *tag ? *tag : "(null)", *date ? *date : "(null)");
+}
+
+
+
 /* Convert a time_t to an RCS format date.  This is mainly for the
    use of "cvs history", because the CVSROOT/history file contains
    time_t format dates; most parts of CVS will want to avoid using
