@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1986-2005 The Free Software Foundation, Inc.
+ * Copyright (C) 1986-2006 The Free Software Foundation, Inc.
  *
  * Portions Copyright (C) 1998-2005 Derek Price, Ximbiot <http://ximbiot.com>,
  *                                  and others.
@@ -26,10 +26,20 @@
  *	command line.
  */
 
-#include "cvs.h"
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
+
+/* GNULIB headers.  */
 #include "save-cwd.h"
 
-
+/* CVS headers.  */
+#include "ignore.h"
+
+#include "cvs.h"
+
+
+
 /* Defines related to the syntax of the modules file.  */
 
 /* Options in modules file.  Note that it is OK to use GNU getopt features;
@@ -120,7 +130,6 @@ my_module (DBM *db, char *mname, enum mtype m_type, char *msg,
     char *mwhere = NULL;
     char *mfile = NULL;
     char *spec_opt = NULL;
-    char *xvalue = NULL;
     int alias = 0;
     datum key, val;
     char *cp;
@@ -728,8 +737,6 @@ module `%s' is a request for a file in a module which is not a directory",
     if (value != NULL)
 	free (value);
 
-    if (xvalue != NULL)
-	free (xvalue);
     return (err);
 }
 
