@@ -1,10 +1,10 @@
 /*
- *    Copyright (c) 2004  Derek Price, Ximbiot <http://ximbiot.com>,
- *                        and the Free Software Foundation
+ *  Copyright (C) 2004, 2006 The Free Software Foundation, Inc.
+ *  Copyright (C) 2004  Derek Price, Ximbiot <http://ximbiot.com>
  *
- *    You may distribute under the terms of the GNU General Public License
- *    as specified in the README file that comes with the CVS source
- *    distribution.
+ *  You may distribute under the terms of the GNU General Public License
+ *  as specified in the README file that comes with the CVS source
+ *  distribution.
  *
  * This is the header file for definitions and functions shared by parseinfo.c
  * with other portions of CVS.
@@ -12,12 +12,25 @@
 #ifndef PARSEINFO_H
 # define PARSEINFO_H
 
+/* ANSI C headers.  */
+#include <stdbool.h>
+#include <stddef.h>	/* Get size_t.  */
+
+/* CVS headers.  */
+#include "root.h"
+#include "verify.h"
+
+
+
 struct config
 {
     void *keywords;
     bool top_level_admin;
     char *lock_dir;
     char *logHistory;
+    char *HistoryLogPath;
+    char *HistorySearchPath;
+    char *TmpDir;
 
     /* Should the logmsg be re-read during the do_verify phase?
      * RereadLogAfterVerify=no|stat|yes
@@ -49,12 +62,33 @@ struct config
 #ifdef PROXY_SUPPORT
     size_t MaxProxyBufferSize;
 #endif /* PROXY_SUPPORT */
+#ifdef SERVER_SUPPORT
+    size_t MinCompressionLevel;
+    size_t MaxCompressionLevel;
+#endif /* SERVER_SUPPORT */
+
+    verify_state VerifyCommits;
+    char *VerifyTemplate;
+    char *OpenPGPTextmode;
+    List *VerifyArgs;
+
 #ifdef PRESERVE_PERMISSIONS_SUPPORT
     bool preserve_perms;
 #endif /* PRESERVE_PERMISSIONS_SUPPORT */
 };
 
+
+
+/***
+ ***
+ ***   CVSROOT/config options
+ ***
+ ***/
+extern struct config *config;
+
+
+
 bool parse_error (const char *, unsigned int);
-struct config *parse_config (const char *);
+struct config *parse_config (const char *, const char *);
 void free_config (struct config *data);
 #endif /* !PARSEINFO_H */
