@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1986-2005 The Free Software Foundation, Inc.
+ * Copyright (C) 1986-2006 The Free Software Foundation, Inc.
  *
  * Portions Copyright (C) 1998-2005 Derek Price, Ximbiot <http://ximbiot.com>,
  *                                  and others.
@@ -11,9 +11,19 @@
  * specified in the README file that comes with the CVS source distribution.
  */
 
-#include "cvs.h"
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
+
+/* Verify interface.  */
+#include "repos.h"
+
+/* GNULIB headers.  */
 #include "getline.h"
 
+/* CVS headers.  */
+#include "cvs.h"
+
 
 
 /* Determine the name of the RCS repository for directory DIR in the
@@ -104,7 +114,7 @@ Name_Repository (const char *dir, const char *update_dir)
      * this function is called, we the contents of CVS/Root have already been
      * compared to original_root and found to match.
      */
-    if (!isabsolute (repos))
+    if (!ISABSOLUTE (repos))
     {
 	char *newrepos;
 
