@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1986-2005 The Free Software Foundation, Inc.
+ * Copyright (C) 1986-2006 The Free Software Foundation, Inc.
  *
  * Portions Copyright (C) 1998-2005 Derek Price, Ximbiot <http://ximbiot.com>,
  *                                  and others.
@@ -17,9 +17,23 @@
  * the modules database, if necessary.
  */
 
-#include "cvs.h"
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
+
+/* GNULIB headers.  */
 #include "save-cwd.h"
 
+/* CVS headers.  */
+#include "classify.h"
+#include "ignore.h"
+#include "recurse.h"
+#include "repos.h"
+
+#include "cvs.h"
+
+
+
 static int rtag_proc (int argc, char **argv, char *xwhere,
 		      char *mwhere, char *mfile, int shorten,
 		      int local_specified, char *mname, char *msg);
@@ -72,10 +86,10 @@ static List *mtlist;
 static const char rtag_opts[] = "+aBbdFflnQqRr:D:";
 static const char *const rtag_usage[] =
 {
-    "Usage: %s %s [-abdFflnR] [-r rev|-D date] tag modules...\n",
+    "Usage: %s %s [-aBbdFflnR] [-r rev|-D date] tag modules...\n",
     "\t-a\tClear tag from removed files that would not otherwise be tagged.\n",
-    "\t-b\tMake the tag a \"branch\" tag, allowing concurrent development.\n",
     "\t-B\tAllows -F and -d to disturb branch tags.  Use with extreme care.\n",
+    "\t-b\tMake the tag a \"branch\" tag, allowing concurrent development.\n",
     "\t-d\tDelete the given tag.\n",
     "\t-F\tMove tag if it already exists.\n",
     "\t-f\tForce a head revision match if tag/date not found.\n",
@@ -92,8 +106,8 @@ static const char tag_opts[] = "+BbcdFflQqRr:D:";
 static const char *const tag_usage[] =
 {
     "Usage: %s %s [-bcdFflR] [-r rev|-D date] tag [files...]\n",
-    "\t-b\tMake the tag a \"branch\" tag, allowing concurrent development.\n",
     "\t-B\tAllows -F and -d to disturb branch tags.  Use with extreme care.\n",
+    "\t-b\tMake the tag a \"branch\" tag, allowing concurrent development.\n",
     "\t-c\tCheck that working files are unmodified.\n",
     "\t-d\tDelete the given tag.\n",
     "\t-F\tMove tag if it already exists.\n",
@@ -240,9 +254,10 @@ cvstag (int argc, char **argv)
 	{
 	    send_files (argc, argv, local, 0,
 
-		    /* I think the -c case is like "cvs status", in
-		       which we really better be correct rather than
-		       being fast; it is just too confusing otherwise.  */
+			/* I think the -c case is like "cvs status", in
+			 * which we really better be correct rather than
+			 * being fast; it is just too confusing otherwise.
+			 */
 			check_uptodate ? 0 : SEND_NO_CONTENTS);
 	    send_file_names (argc, argv, SEND_EXPAND_WILD);
 	    send_to_server ("tag\012", 0);
@@ -492,7 +507,7 @@ rtag_proc (int argc, char **argv, char *xwhere, char *mwhere, char *mfile,
 	}
 	/* End section which is identical to patch_proc.  */
 
-	if (delete_flag || attic_too || (force_tag_match && numtag))
+	if (delete_flag || force_tag_move || attic_too || numtag)
 	    which = W_REPOS | W_ATTIC;
 	else
 	    which = W_REPOS;
@@ -1532,7 +1547,13 @@ static void add_to_val_tags (const char *name)
     val_tags_lock (current_parsed_root->directory);
 
     /* Check for presence again since we have a lock now.  */
-    if (is_in_val_tags (&db, name)) return;
+    if (is_in_val_tags (&db, name))
+    {
+	clear_val_tags_lock ();
+	if (db)
+	    dbm_close (db);
+	return;
+    }
 
     /* Casting out const should be safe here - input datums are not
      * written to by the myndbm functions.
