@@ -12,6 +12,7 @@
  */
 
 #include "cvs.h"
+#include "lstat.h"
 
 #ifdef SERVER_SUPPORT
 static void time_stamp_server (const char *, Vers_TS *, Entnode *);
@@ -33,12 +34,16 @@ static void time_stamp_server (const char *, Vers_TS *, Entnode *);
  *   set_time		If set, set the last modification time of the user file
  *			specified by FINFO to the checkin time of RET->vn_rcs.
  *
+ * OUTPUTS
+ *   finfo		FINFO->RCS will be updated if it was empty and RCS data
+ *			is found and parsed.
+ *
  * RETURNS
  *   Vers_TS structure for FINFO.
  */
 Vers_TS *
-Version_TS (struct file_info *finfo, char *options, char *tag, char *date,
-            int force_tag_match, int set_time)
+Version_TS (struct file_info *finfo, const char *options, const char *tag,
+	    const char *date, int force_tag_match, int set_time)
 {
     Node *p;
     RCSNode *rcsdata;
@@ -47,7 +52,12 @@ Version_TS (struct file_info *finfo, char *options, char *tag, char *date,
     Entnode *entdata;
     char *rcsexpand = NULL;
 
+    TRACE (TRACE_FUNCTION, "Version_TS (%s, %s, %s, %s, %d, %d)",
+	   finfo->fullname, TRACE_NULL (options), TRACE_NULL (tag),
+	   TRACE_NULL (date), force_tag_match, set_time);
+
     /* get a new Vers_TS struct */
+
     vers_ts = xmalloc (sizeof (Vers_TS));
     memset (vers_ts, 0, sizeof (*vers_ts));
 
@@ -69,6 +79,7 @@ Version_TS (struct file_info *finfo, char *options, char *tag, char *date,
 
     if (p == NULL)
     {
+	TRACE (TRACE_DATA, "Version_TS: No entries data found.");
 	entdata = NULL;
     }
     else
@@ -188,6 +199,11 @@ Version_TS (struct file_info *finfo, char *options, char *tag, char *date,
     {
 	/* squirrel away the rcsdata pointer for others */
 	vers_ts->srcfile = rcsdata;
+	if (!finfo->rcs)
+	{
+	    rcsdata->refcount++;
+	    finfo->rcs = rcsdata;
+	}
 
 	if (vers_ts->tag && strcmp (vers_ts->tag, TAG_BASE) == 0)
 	{
@@ -269,7 +285,7 @@ Version_TS (struct file_info *finfo, char *options, char *tag, char *date,
 	    vers_ts->ts_user = time_stamp (finfo->file);
     }
 
-    return (vers_ts);
+    return vers_ts;
 }
 
 
@@ -288,7 +304,13 @@ time_stamp_server (const char *file, Vers_TS *vers_ts, Entnode *entdata)
     struct stat sb;
     char *cp;
 
-    if (CVS_LSTAT (file, &sb) < 0)
+    TRACE (TRACE_FUNCTION, "time_stamp_server (%s, %s, %s, %s)",
+	   file,
+	   entdata && entdata->version ? entdata->version : "(null)",
+	   entdata && entdata->timestamp ? entdata->timestamp : "(null)",
+	   entdata && entdata->conflict ? entdata->conflict : "(null)");
+
+    if (lstat (file, &sb) < 0)
     {
 	if (! existence_error (errno))
 	    error (1, errno, "cannot stat temp file");
@@ -307,6 +329,13 @@ time_stamp_server (const char *file, Vers_TS *vers_ts, Entnode *entdata)
 		 && entdata->timestamp[0] == '='
 		 && entdata->timestamp[1] == '\0')
 	    mark_unchanged (vers_ts);
+	else if (entdata->conflict
+		 && entdata->conflict[0] == '=')
+	{
+	    /* These need matching content that doesn't match ->ts_rcs.  */
+	    vers_ts->ts_user = xstrdup ("Is-modified");
+	    vers_ts->ts_conflict = xstrdup ("Is-modified");
+	}
 	else if (entdata->timestamp
 		 && (entdata->timestamp[0] == 'M'
 		     || entdata->timestamp[0] == 'D')
@@ -386,7 +415,7 @@ unix_time_stamp (const char *file)
     struct stat sb;
     time_t mtime = 0L;
 
-    if (!CVS_LSTAT (file, &sb))
+    if (!lstat (file, &sb))
     {
 	mtime = sb.st_mtime;
     }
@@ -394,7 +423,7 @@ unix_time_stamp (const char *file)
     /* If it's a symlink, return whichever is the newest mtime of
        the link and its target, for safety.
     */
-    if (!CVS_STAT (file, &sb))
+    if (!stat (file, &sb))
     {
         if (mtime < sb.st_mtime)
 	    mtime = sb.st_mtime;
