@@ -1,16 +1,32 @@
-/* This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
+/*
+ * Copyright (C) 2006 The Free Software Foundation, Inc.
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2, or (at your option)
+ * any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ */
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.  */
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
 
-#include "cvs.h"
+/* Verify interface.  */
+#include "wrapper.h"
+
+/* GNULIB headers.  */
 #include "getline.h"
 
+/* CVS headers.  */
+#include "cvs.h"
+
+
+
 /*
   Original Author:  athan@morgan.com <Andrew C. Athan> 2/1/94
   Modified By:      vdemarco@bou.shl.com
@@ -93,9 +109,7 @@ void wrap_setup(void)
     else
         wrap_setup_already_done = 1;
 
-#ifdef CLIENT_SUPPORT
     if (!current_parsed_root->isremote)
-#endif
     {
 	char *file;
 
@@ -394,7 +408,8 @@ wrap_add (char *line, int isTemp)
 	switch(opt){
 	case 'f':
 	    /* Before this is reenabled, need to address the problem in
-	       commit.c (see http://www.cvshome.org/docs/infowrapper.html).  */
+	       commit.c (see
+	       <http://ximbiot.com/cvs/cvshome/docs/infowrapper.html>).  */
 	    error (1, 0,
 		   "-t/-f wrappers not supported by this version of CVS");
 
@@ -402,13 +417,16 @@ wrap_add (char *line, int isTemp)
 		free(e.fromcvsFilter);
 	    /* FIXME: error message should say where the bad value
 	       came from.  */
-	    e.fromcvsFilter=expand_path (temp, "<wrapper>", 0, 0);
+	    e.fromcvsFilter =
+	      expand_path (temp, current_parsed_root->directory, false,
+			   "<wrapper>", 0);
             if (!e.fromcvsFilter)
 		error (1, 0, "Correct above errors first");
 	    break;
 	case 't':
 	    /* Before this is reenabled, need to address the problem in
-	       commit.c (see http://www.cvshome.org/docs/infowrapper.html).  */
+	       commit.c (see
+	       <http://ximbiot.com/cvs/cvshome/docs/infowrapper.html>).  */
 	    error (1, 0,
 		   "-t/-f wrappers not supported by this version of CVS");
 
@@ -416,7 +434,8 @@ wrap_add (char *line, int isTemp)
 		free(e.tocvsFilter);
 	    /* FIXME: error message should say where the bad value
 	       came from.  */
-	    e.tocvsFilter=expand_path (temp, "<wrapper>", 0, 0);
+	    e.tocvsFilter = expand_path (temp, current_parsed_root->directory,
+					 false, "<wrapper>", 0);
             if (!e.tocvsFilter)
 		error (1, 0, "Correct above errors first");
 	    break;
