@@ -19,14 +19,20 @@
 
 #include <assert.h>
 #include <io.h>
-#include <windows.h>
+#include <sys/socket.h>  /* This does: #include <windows.h> */
 
 #include "cvs.h"
+#include "setenv.h"
 
 #include "JmgStat.h"
 
-static int deep_remove_dir( const char *path );
+#undef mkdir
 
+static int deep_remove_dir (const char *path);
+
+/*
+ * Copies FROM to TO.  Ignores NOEXEC.
+ */
 /* Copies "from" to "to".  Note that the functionality here is similar
    to the win32 function CopyFile, but (1) we copy LastAccessTime and
    CopyFile doesn't, (2) we set file attributes to the default set by
@@ -35,21 +41,14 @@ static int deep_remove_dir( const char *path );
    is some reason they should be changed (this would need more
    investigation).  */
 void
-copy_file (from, to)
-    const char *from;
-    const char *to;
+force_copy_file (const char *from, const char *to )
 {
     struct stat sb;
     struct utimbuf t;
     int fdin, fdout;
 
-    if (trace)
-#ifdef SERVER_SUPPORT
-	(void) fprintf (stderr, "%c-> copy(%s,%s)\n",
-			(server_active) ? 'S' : ' ', from, to);
-#else
-	(void) fprintf (stderr, "-> copy(%s,%s)\n", from, to);
-#endif
+    TRACE (TRACE_FUNCTION, "copy (%s, %s)", from, to);
+
     if (noexec)
 	return;
 
@@ -65,9 +64,9 @@ copy_file (from, to)
 	char buf[BUFSIZ];
 	int n;
 
-	for (;;) 
+	for (;;)
 	{
-	    n = read (fdin, buf, sizeof(buf));
+	    n = read (fdin, buf, sizeof (buf));
 	    if (n == -1)
 	    {
 #ifdef EINTR
@@ -76,21 +75,21 @@ copy_file (from, to)
 #endif
 		error (1, errno, "cannot read file %s for copying", from);
 	    }
-            else if (n == 0) 
+            else if (n == 0)
 		break;
-  
-	    if (write(fdout, buf, n) != n) {
+
+	    if (write (fdout, buf, n) != n) {
 		error (1, errno, "cannot write file %s for copying", to);
 	    }
 	}
 
 #ifdef HAVE_FSYNC
-	if (fsync (fdout)) 
+	if (fsync (fdout))
 	    error (1, errno, "cannot fsync file %s after copying", to);
 #endif
     }
 
-    if (close (fdin) < 0) 
+    if (close (fdin) < 0)
 	error (0, errno, "cannot close %s", from);
     if (close (fdout) < 0)
 	error (1, errno, "cannot close %s", to);
@@ -102,81 +101,148 @@ copy_file (from, to)
     (void) utime (to, &t);
 }
 
+/*
+ * Copies FROM to TO.  Honors NOEXEC.
+ */
+void
+copy_file (const char *from, const char *to)
+{
+    TRACE (TRACE_FUNCTION, "copy (%s, %s)", from, to);
+    if (noexec) return;
+    force_copy_file (from, to);
+}
+
+
+static char *tmpdir_env;
+/*
+ * Return seperator (\) terminated path to system temporary directory.
+ */
+const char *
+get_system_temp_dir (void)
+{
+	if (! tmpdir_env)
+	{
+		DWORD dwBufferSize, dwReturn;
+
+		dwReturn = 0;
+		dwBufferSize = 64;
+		do {
+			if (dwReturn >= dwBufferSize)
+			{
+				dwBufferSize = dwReturn + 4;
+			}
+
+			tmpdir_env = xrealloc (tmpdir_env, dwBufferSize);
+			if (tmpdir_env)
+			{
+				dwReturn = GetTempPath (dwBufferSize, tmpdir_env);
+				if (dwReturn <= 0)
+				{
+					free (tmpdir_env);
+					tmpdir_env = NULL;
+				}
+			}
+		} while (tmpdir_env && dwReturn >= dwBufferSize);
+	}
+
+	return tmpdir_env;
+}
+
+
+void
+push_env_temp_dir (void)
+{
+	const char *tmpdir = get_cvs_tmp_dir ();
+
+	if (tmpdir_env && strcmp (tmpdir_env, tmpdir))
+		setenv ("TMP", tmpdir, 1);
+}
+
+
+
 /* FIXME-krp: these functions would benefit from caching the char * &
    stat buf.  */
 
 /*
- * Returns non-zero if the argument file is a directory, or is a symbolic
+ * Returns true if the argument file is a directory, or is a symbolic
  * link which points to a directory.
  */
-int
-isdir (file)
-    const char *file;
+bool
+isdir (const char *file)
 {
     struct stat sb;
 
     if (stat (file, &sb) < 0)
-	return (0);
-    return (S_ISDIR (sb.st_mode));
+	return false;
+    return S_ISDIR (sb.st_mode);
 }
 
+
+
 /*
- * Returns non-zero if the argument file is a symbolic link.
+ * Returns 0 if the argument file is not a symbolic link.
+ * Returns size of the link if it is a symbolic link.
+ *
+ * FIXME: Is there a good reason that the off_t specified by POSIX for st_size
+ *        (http://www.opengroup.org/susv3xbd/sys/stat.h.html) is converted to
+ *        ssize_t here?  rcs.h uses off_t, so it's not because off_t isn't
+ *        portable.
  */
-int
-islink (file)
-    const char *file;
+ssize_t
+islink (const char *file)
 {
+    ssize_t retsize = 0;
 #ifdef S_ISLNK
     struct stat sb;
 
-    if (lstat (file, &sb) < 0)
-	return (0);
-    return (S_ISLNK (sb.st_mode));
-#else
-    return (0);
+    if ((lstat (file, &sb) >= 0) && S_ISLNK (sb.st_mode))
+	retsize = sb.st_size;
 #endif
+    return retsize;
 }
 
+
+
 /*
- * Returns non-zero if the argument file exists.
+ * Returns true if the argument file exists.
  */
-int
-isfile (file)
-    const char *file;
+bool
+isfile (const char *file)
 {
-    return isaccessible(file, F_OK);
+    return isaccessible (file, F_OK);
 }
 
+
+
 /*
  * Returns non-zero if the argument file is readable.
  */
-int
-isreadable (file)
-    const char *file;
+bool
+isreadable (const char *file)
 {
-    return isaccessible(file, R_OK);
+    return isaccessible (file, R_OK);
 }
 
+
+
 /*
  * Returns non-zero if the argument file is writable.
  */
-int
-iswritable (file)
-    const char *file;
+bool
+iswritable (const char *file)
 {
-    return isaccessible(file, W_OK);
+    return isaccessible (file, W_OK);
 }
 
+
+
 /*
- * Returns non-zero if the argument file is accessable according to
+ * Returns true if the argument file is accessable according to
  * mode.  If compiled with SETXID_SUPPORT also works if cvs has setxid
  * bits set.
  */
-int
-isaccessible (file, mode)
-    const char *file;
-    const int mode;
+bool
+isaccessible (const char *file, const int mode)
 {
 #ifdef SETXID_SUPPORT
     struct stat sb;
@@ -184,19 +250,22 @@ isaccessible (file, mode)
     int gmask = 0;
     int omask = 0;
     int uid;
-    
-    if (stat(file, &sb) == -1)
-	return 0;
-    if (mode == F_OK)
-	return 1;
 
-    uid = geteuid();
+    if (stat (file, &sb) == -1)
+	return false;
+    if (mode == F_OK)
+	return true;
+
+    uid = geteuid ();
     if (uid == 0)		/* superuser */
     {
-	if (mode & X_OK)
-	    return sb.st_mode & (S_IXUSR|S_IXGRP|S_IXOTH);
-	else
-	    return 1;
+	if (!(mode & X_OK) || (sb.st_mode & (S_IXUSR|S_IXGRP|S_IXOTH)))
+	    return true;
+
+#ifdef EACCES
+	errno = EACCES;
+#endif
+	return false;
     }
 	
     if (mode & R_OK)
@@ -220,36 +289,22 @@ isaccessible (file, mode)
 
     if (sb.st_uid == uid)
 	return (sb.st_mode & umask) == umask;
-    else if (sb.st_gid == getegid())
+    else if (sb.st_gid == getegid ())
 	return (sb.st_mode & gmask) == gmask;
     else
 	return (sb.st_mode & omask) == omask;
-#else
-    return access(file, mode) == 0;
-#endif
+#else /* !SETXID_SUPPORT */
+    return access (file, mode) == 0;
+#endif /* SETXID_SUPPORT */
 }
 
-/*
- * Open a file and die if it fails
- */
-FILE *
-open_file (name, mode)
-    const char *name;
-    const char *mode;
-{
-    FILE *fp;
 
-    if ((fp = fopen (name, mode)) == NULL)
-	error (1, errno, "cannot open %s", name);
-    return (fp);
-}
 
 /*
  * Make a directory and die if it fails
  */
 void
-make_directory (name)
-    const char *name;
+make_directory (const char *name)
 {
     struct stat sb;
 
@@ -264,8 +319,7 @@ make_directory (name)
  * goes wrong.
  */
 void
-make_directories (name)
-    const char *name;
+make_directories (const char *name)
 {
     char *cp;
 
@@ -293,8 +347,7 @@ make_directories (name)
    other errors.  Returns 0 if directory was created; 1 if it already
    existed.  */
 int
-mkdir_if_needed (name)
-    const char *name;
+mkdir_if_needed (const char *name)
 {
     if (mkdir (name) < 0)
     {
@@ -315,16 +368,15 @@ mkdir_if_needed (name)
     }
     return 0;
 }
-
 /*
  * Change the mode of a file, either adding write permissions, or removing
  * all write permissions.  Adding write permissions honors the current umask
  * setting.
  */
-void
-xchmod (fname, writable)
+static void
+ixchmod (fname, writable)
     const char *fname;
-    int writable;
+    bool writable;
 {
     struct stat sb;
     mode_t mode, oumask;
@@ -348,13 +400,8 @@ xchmod (fname, writable)
 	mode = sb.st_mode & ~(S_IWRITE | S_IWGRP | S_IWOTH);
     }
 
-    if (trace)
-#ifdef SERVER_SUPPORT
-	(void) fprintf (stderr, "%c-> chmod(%s,%o)\n",
-			(server_active) ? 'S' : ' ', fname, mode);
-#else
-	(void) fprintf (stderr, "-> chmod(%s,%o)\n", fname, mode);
-#endif
+    TRACE (TRACE_FUNCTION, "chmod(%s,%o)", fname, (unsigned int) mode);
+
     if (noexec)
 	return;
 
@@ -362,6 +409,17 @@ xchmod (fname, writable)
 	error (0, errno, "cannot change mode of file %s", fname);
 }
 
+/* See description for ixchmod.  Ignores NOEXEC.  */
+void
+force_xchmod (const char *fname, bool writable)
+{
+    ixchmod (fname, writable, false);
+}
+
+void xchmod(const char *fname, bool writable)
+{
+	ixchmod(fname, writable, noexec);
+}
 
 /* Rename for NT which works for read only files.  Apparently if we are
    accessing FROM and TO via a Novell network, this is an issue.  */
@@ -398,71 +456,62 @@ wnt_rename (from, to)
     return result;
 }
 
+
+
 /*
  * Rename a file and die if it fails
  */
 void
-rename_file (from, to)
-    const char *from;
-    const char *to;
+rename_file (const char *from, const char *to)
 {
-    if (trace)
-#ifdef SERVER_SUPPORT
-	(void) fprintf (stderr, "%c-> rename(%s,%s)\n",
-			(server_active) ? 'S' : ' ', from, to);
-#else
-	(void) fprintf (stderr, "-> rename(%s,%s)\n", from, to);
-#endif
+    TRACE (TRACE_FUNCTION, "rename(%s,%s)", from, to);
+
     if (noexec)
 	return;
 
     /* Win32 unlink is stupid --- it fails if the file is read-only  */
-    chmod(to, S_IWRITE);
-    unlink(to);
+    chmod (to, S_IWRITE);
+    unlink (to);
     if (CVS_RENAME (from, to) < 0)
 	error (1, errno, "cannot rename file %s to %s", from, to);
 }
 
+
+
 /*
  * unlink a file, if possible.
  */
 int
-unlink_file (f)
-    const char *f;
+unlink_file (const char *f)
 {
-    if (trace)
-#ifdef SERVER_SUPPORT
-	(void) fprintf (stderr, "%c-> unlink(%s)\n",
-			(server_active) ? 'S' : ' ', f);
-#else
-	(void) fprintf (stderr, "-> unlink(%s)\n", f);
-#endif
+    TRACE (TRACE_FUNCTION, "unlink_file(%s)", f);
+
     if (noexec)
-	return (0);
+	return 0;
 
     /* Win32 unlink is stupid - it fails if the file is read-only */
     chmod (f, _S_IWRITE);
-    return (unlink (f));
+    return unlink (f);
 }
 
+
+
 /*
  * Unlink a file or dir, if possible.  If it is a directory do a deep
  * removal of all of the files in the directory.  Return -1 on error
  * (in which case errno is set).
  */
 int
-unlink_file_dir (f)
-    const char *f;
+unlink_file_dir (const char *f)
 {
-    if (trace)
-#ifdef SERVER_SUPPORT
-	(void) fprintf (stderr, "%c-> unlink_file_dir(%s)\n",
-			(server_active) ? 'S' : ' ', f);
-#else
-	(void) fprintf (stderr, "-> unlink_file_dir(%s)\n", f);
-#endif
+    /* This is called by the server parent process in contexts where
+       it is not OK to send output (e.g. after we sent "ok" to the
+       client).  */
+    if (!server_active)
+	TRACE (TRACE_FUNCTION, "unlink_file_dir(%s)", f);
+
     if (noexec)
-	return (0);
+	return 0;
 
     /* Win32 unlink is stupid - it fails if the file is read-only */
     chmod (f, _S_IWRITE);
@@ -484,13 +533,14 @@ unlink_file_dir (f)
     return 0;
 }
 
+
+
 /* Remove a directory and everything it contains.  Returns 0 for
  * success, -1 for failure (in which case errno is set).
  */
 
 static int
-deep_remove_dir (path)
-    const char *path;
+deep_remove_dir (const char *path)
 {
     DIR		  *dirp;
     struct dirent *dp;
@@ -558,19 +608,18 @@ deep_remove_dir (path)
     return 0;
 }
 
+
+
 /* Read NCHARS bytes from descriptor FD into BUF.
    Return the number of characters successfully read.
    The number returned is always NCHARS unless end-of-file or error.  */
 static size_t
-block_read (fd, buf, nchars)
-    int fd;
-    char *buf;
-    size_t nchars;
+block_read (int fd, char *buf, size_t nchars)
 {
     char *bp = buf;
     size_t nread;
 
-    do 
+    do
     {
 	nread = read (fd, bp, nchars);
 	if (nread == (size_t)-1)
@@ -583,23 +632,21 @@ block_read (fd, buf, nchars)
 	}
 
 	if (nread == 0)
-	    break; 
+	    break;
 
 	bp += nread;
 	nchars -= nread;
     } while (nchars != 0);
 
     return bp - buf;
-} 
+}
+
 
-    
 /*
  * Compare "file1" to "file2". Return non-zero if they don't compare exactly.
  */
 int
-xcmp (file1, file2)
-    const char *file1;
-    const char *file2;
+xcmp (const char *file1, const char *file2)
 {
     char *buf1, *buf2;
     struct stat sb1, sb2;
@@ -615,7 +662,7 @@ xcmp (file1, file2)
     if (fstat (fd2, &sb2) < 0)
 	error (1, errno, "cannot fstat %s", file2);
 
-    /* A generic file compare routine might compare st_dev & st_ino here 
+    /* A generic file compare routine might compare st_dev & st_ino here
        to see if the two files being compared are actually the same file.
        But that won't happen in CVS, so we won't bother. */
 
@@ -634,7 +681,7 @@ xcmp (file1, file2)
 	buf1 = xmalloc (buf_size);
 	buf2 = xmalloc (buf_size);
 
-	do 
+	do
 	{
 	    read1 = block_read (fd1, buf1, buf_size);
 	    if (read1 == (size_t)-1)
@@ -646,7 +693,7 @@ xcmp (file1, file2)
 
 	    /* assert (read1 == read2); */
 
-	    ret = memcmp(buf1, buf2, read1);
+	    ret = memcmp (buf1, buf2, read1);
 	} while (ret == 0 && read1 == buf_size);
 
 	free (buf1);
@@ -669,7 +716,7 @@ xcmp (file1, file2)
  * now.
  */
 char *
-cvs_temp_name ()
+cvs_temp_name (void)
 {
     char *fn;
     FILE *fp;
@@ -705,8 +752,7 @@ cvs_temp_name ()
 /* FIXME: This should use the mkstemp() function from the lib/mkstemp.c file
  * from the GNULIB project.
  */
-FILE *cvs_temp_file (filename)
-    char **filename;
+FILE *cvs_temp_file (char ** filename)
 {
     char *fn;
     FILE *fp;
@@ -718,9 +764,14 @@ FILE *cvs_temp_file (filename)
 
     /* assert (filename != NULL); */
 
-    fn = _tempnam (Tmpdir, "cvs");
+    fn = _tempnam (getenv ("TEMP"), "cvs");
     if (fn == NULL) fp = NULL;
-    else if ((fp = CVS_FOPEN (fn, "w+")) == NULL) free (fn);
+    else
+    if ((fp = CVS_FOPEN (fn, "w+")) == NULL)
+    {
+        free (fn);
+        fn = NULL;
+    }
 
     /* tempnam returns a pointer to a newly malloc'd string, so there's
      * no need for a xstrdup
@@ -732,65 +783,6 @@ FILE *cvs_temp_file (filename)
 
 
 
-/* char *
- * xresolvepath ( const char *path )
- *
- * Like xreadlink(), but resolve all links in a path.
- *
- * INPUTS
- *  path	The original path.
- *
- * RETURNS
- *  The path with any symbolic links expanded.
- *
- * ERRORS
- *  This function exits with a fatal error if it fails to read the link for
- *  any reason.
- */
-char *
-xresolvepath ( path )
-    const char *path;
-{
-    char *hardpath;
-    char *owd;
-
-    /* assert ( isdir ( path ) ); */
-
-    /* FIXME - If HAVE_READLINK is defined, we should probably walk the path
-     * bit by bit calling xreadlink().
-     */
-
-    owd = xgetcwd ();
-    if ( CVS_CHDIR ( path ) < 0)
-	error ( 1, errno, "cannot chdir to %s", path );
-    if ((hardpath = xgetcwd ()) == NULL)
-	error (1, errno, "cannot readlink %s", hardpath);
-    if ( CVS_CHDIR ( owd ) < 0)
-	error ( 1, errno, "cannot chdir to %s", owd );
-    free (owd);
-    return hardpath;
-}
-
-
-
-/* Return a pointer into PATH's last component.  */
-const char *
-last_component (const char *path)
-{
-    const char *scan;
-    const char *last = 0;
-
-    for (scan = path; *scan; scan++)
-        if (ISSLASH (*scan))
-	    last = scan;
-
-    if (last && (last != path))
-        return last + 1;
-    else
-        return path;
-}
-
-
 /* NT has two evironment variables, HOMEPATH and HOMEDRIVE, which,
    when combined as ${HOMEDRIVE}${HOMEPATH}, give the unix equivalent
    of HOME.  Some NT users are just too unixy, though, and set the
@@ -818,25 +810,16 @@ last_component (const char *path)
    said to be set to c:\users\default by default.  */
 
 char *
-get_homedir ()
+get_homedir (void)
 {
-    static char *pathbuf;
-    char *hd, *hp;
+    char *homedir;
 
-    if (pathbuf != NULL)
-	return pathbuf;
-    else if ((hd = getenv ("HOME")))
-	return hd;
-    else if ((hd = getenv ("HOMEDRIVE")) && (hp = getenv ("HOMEPATH")))
-    {
-	pathbuf = xmalloc (strlen (hd) + strlen (hp) + 5);
-	strcpy (pathbuf, hd);
-	strcat (pathbuf, hp);
+    homedir = getenv ("HOME");
 
-	return pathbuf;
-    }
-    else
-	return NULL;
+    if (homedir == NULL)
+	homedir = woe32_home_dir ();
+
+    return homedir;
 }
 
 /* Compose a path to a file in the home directory.  This is necessary because
@@ -855,7 +838,7 @@ strcat_filename_onto_homedir (dir, file)
     const char *dir;
     const char *file;
 {
-    char *path = xmalloc (strlen (dir) + 1 + strlen(file) + 1);
+    char *path = xmalloc (strlen (dir) + 1 + strlen (file) + 1);
     sprintf (path, "%s\\%s", dir, file);
     return path;
 }
@@ -890,13 +873,12 @@ expand_wild (argc, argv, pargc, pargv)
 	char *last_forw_slash, *last_back_slash, *end_of_dirname;
 	int dirname_length = 0;
 
-	/* FIXME: If argv[i] is ".", this code will expand it to the
-	   name of the current directory in its parent directory which
-	   will cause start_recursion to do all manner of strange things
-	   with it (culminating in an error).  This breaks "cvs co .".
-	   As nearly as I can guess, this bug has existed since
-	   expand_wild was first created.  At least, it is in CVS 1.9 (I
-	   just tried it).  */
+	if ( strcmp (argv[i], ".") == 0 )
+	{
+	    new_argv[new_argc] = (char *) xmalloc (2);
+	    strcpy (new_argv[ new_argc++ ], ".");
+	    continue;
+	}
 
 	/* FindFirstFile doesn't return pathnames, so we have to do
 	   this ourselves.  Luckily, it's no big deal, since globbing
@@ -907,7 +889,7 @@ expand_wild (argc, argv, pargc, pargv)
 
 	/* Win32 can handle both forward and backward slashes as
            filenames -- check for both. */
-	     
+
 	last_forw_slash = strrchr (argv[i], '/');
 	last_back_slash = strrchr (argv[i], '\\');
 
@@ -936,7 +918,8 @@ expand_wild (argc, argv, pargc, pargv)
 		if (new_argc == max_new_argc)
 		{
 		    max_new_argc *= 2;
-		    new_argv = xrealloc (new_argv, max_new_argc * sizeof (char *));
+		    new_argv = xrealloc (new_argv,
+					 max_new_argc * sizeof (char *));
 		}
 	    }
 	    else
@@ -980,7 +963,8 @@ expand_wild (argc, argv, pargc, pargv)
 		if (new_argc == max_new_argc)
 		{
 		    max_new_argc *= 2;
-		    new_argv = xrealloc (new_argv, max_new_argc * sizeof (char *));
+		    new_argv = xrealloc (new_argv,
+					 max_new_argc * sizeof (char *));
 		}
 		if (!FindNextFile (h, &fdata))
 		{
@@ -998,9 +982,23 @@ expand_wild (argc, argv, pargc, pargv)
     *pargv = new_argv;
 }
 
-static void check_statbuf (const char *file, struct stat *sb)
+
+
+/* undo config.h stat macro */
+#undef stat
+extern int stat (const char *file, struct wnt_stat *sb);
+
+/* see config.h stat macro */
+int
+wnt_stat (const char *file, struct wnt_stat *sb)
 {
-    /* Win32 processes file times in a 64 bit format
+    int retval;
+
+    retval = stat (file, sb);
+    if (retval < 0)
+		return retval;
+
+	/* Win32 processes file times in a 64 bit format
        (see Win32 functions SetFileTime and GetFileTime).
        If the file time on a file doesn't fit into the
        32 bit time_t format, then stat will set that time
@@ -1012,40 +1010,16 @@ static void check_statbuf (const char *file, struct stat *sb)
        on Win32 via GetFileTime, but that would be a lot of
        hair and I'm not sure there is much payoff.  */
     if (sb->st_mtime == (time_t) -1)
-	error (1, 0, "invalid modification time for %s", file);
+		error (1, 0, "invalid modification time for %s", file);
     if (sb->st_ctime == (time_t) -1)
 	/* I'm not sure what this means on windows.  It
 	   might be a creation time (unlike unix)....  */
-	error (1, 0, "invalid ctime for %s", file);
+		error (1, 0, "invalid ctime for %s", file);
     if (sb->st_atime == (time_t) -1)
-	error (1, 0, "invalid access time for %s", file);
+		error (1, 0, "invalid access time for %s", file);
 
     if (!GetUTCFileModTime (file, &sb->st_mtime))
-	error (1, 0, "Failed to retrieve modification time for %s", file);
-}
+		error (1, 0, "Failed to retrieve modification time for %s", file);
 
-/* see CVS_STAT */
-int
-wnt_stat (const char *file, struct stat *sb)
-{
-    int retval;
-
-    retval = stat (file, sb);
-    if (retval < 0)
-	return retval;
-    check_statbuf (file, sb);
-    return retval;
-}
-
-/* see CVS_LSTAT */
-int
-wnt_lstat (const char *file, struct stat *sb)
-{
-    int retval;
-
-    retval = lstat (file, sb);
-    if (retval < 0)
-	return retval;
-    check_statbuf (file, sb);
     return retval;
 }
